@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #ifndef __XDEBUG_CODE_COVERAGE_PRIVATE_H__
@@ -42,14 +40,12 @@ typedef struct xdebug_coverage_function {
 #define XG_COV(v)      (XG(globals.coverage.v))
 #define XINI_COV(v)    (XG(settings.coverage.v))
 
-xdebug_coverage_file *xdebug_coverage_file_ctor(char *filename);
+xdebug_coverage_file *xdebug_coverage_file_ctor(zend_string *filename);
 
 xdebug_coverage_function *xdebug_coverage_function_ctor(char *function_name);
 void xdebug_coverage_function_dtor(void *data);
 void xdebug_code_coverage_start_of_function(zend_op_array *op_array, char *function_name);
-void xdebug_code_coverage_end_of_function(zend_op_array *op_array, char *file_name, char *function_name);
-
-int xdebug_common_override_handler(zend_execute_data *execute_data);
+void xdebug_code_coverage_end_of_function(zend_op_array *op_array, zend_string *file_name, char *function_name);
 
 PHP_FUNCTION(xdebug_start_code_coverage);
 PHP_FUNCTION(xdebug_stop_code_coverage);
