@@ -53,8 +53,6 @@
 
 #include "debugger_private.h"
 #include "handler_dbgp.h"
-#include "lib/crc32.h"
-#include "lib/log.h"
 
 ZEND_EXTERN_MODULE_GLOBALS(xdebug)
 
@@ -65,21 +63,21 @@ static int xdebug_create_socket_unix(const char *path)
 	int sockfd;
 
 	if ((sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) == SOCK_ERR) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "UNIX", "Creating socket for 'unix://%s', socket: %s.", path, strerror(errno));
+		XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for 'unix://%s', socket: %s.\n", path, strerror(errno));
 		return SOCK_ERR;
 	}
 
 	sa.sun_family = AF_UNIX;
 	strncpy(sa.sun_path, path, sizeof(sa.sun_path) - 1);
 	if (connect(sockfd, (struct sockaddr*)&sa, sizeof(sa)) < 0) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "UNIX", "Creating socket for 'unix://%s', connect: %s.", path, strerror(errno));
+		XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for 'unix://%s', connect: %s.\n", path, strerror(errno));
 		SCLOSE(sockfd);
 		return (errno == EACCES) ? SOCK_ACCESS_ERR : SOCK_ERR;
 	}
 
 	/* Prevent the socket from being inherited by exec'd children */
 	if (fcntl(sockfd, F_SETFD, FD_CLOEXEC) < 0) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "UNIX", "Creating socket for 'unix://%s', fcntl(FD_CLOEXEC): %s.", path, strerror(errno));
+		XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for 'unix://%s', fcntl(FD_CLOEXEC): %s.\n", path, strerror(errno));
 	}
 
 	return sockfd;
@@ -89,12 +87,12 @@ static int xdebug_create_socket_unix(const char *path)
 #if !WIN32 && !WINNT
 
 /* For OSX and FreeBSD */
-# if !defined(SOL_TCP) && defined(IPPROTO_TCP)
-#  define SOL_TCP IPPROTO_TCP
-# endif
-# if !defined(TCP_KEEPIDLE) && defined(TCP_KEEPALIVE)
-#  define TCP_KEEPIDLE TCP_KEEPALIVE
-# endif
+#if !defined(SOL_TCP) && defined(IPPROTO_TCP) && (defined(__APPLE__) || defined(__FreeBSD__))
+# define SOL_TCP IPPROTO_TCP
+#endif
+#if !defined(TCP_KEEPIDLE) && defined(TCP_KEEPALIVE) && defined(__APPLE__)
+# define TCP_KEEPIDLE TCP_KEEPALIVE
+#endif
 
 void set_keepalive_options(int fd)
 {
@@ -104,38 +102,28 @@ void set_keepalive_options(int fd)
 
 	ret = setsockopt(fd, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen);
 	if (ret) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "KEEPALIVE", "Could not set SO_KEEPALIVE: %s.", strerror(errno));
 		return;
 	}
 
-# if defined(TCP_KEEPIDLE)
 	optval = 600;
 	ret = setsockopt(fd, SOL_TCP, TCP_KEEPIDLE, &optval, optlen);
 	if (ret) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "KEEPALIVE", "Could not set TCP_KEEPIDLE to %d: %s.", optval, strerror(errno));
 		return;
 	}
-# endif
 
-# if defined(TCP_KEEPCNT)
 	optval = 20;
 	ret = setsockopt(fd, SOL_TCP, TCP_KEEPCNT, &optval, optlen);
 	if (ret) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "KEEPALIVE", "Could not set TCP_KEEPCNT to %d: %s.", optval, strerror(errno));
 		return;
 	}
-# endif
 
-# if defined(TCP_KEEPINTVL)
 	optval = 60;
 	ret = setsockopt(fd, SOL_TCP, TCP_KEEPINTVL, &optval, optlen);
 	if (ret) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "KEEPALIVE", "Could not set TCP_KEEPINTVL to %d: %s.", optval, strerror(errno));
 		return;
 	}
-# endif
 }
-#endif  // !WIN32 && !WINNT
+#endif
 
 static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 {
@@ -166,7 +154,7 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 
 	if (!strncmp(hostname, "unix://", strlen("unix://"))) {
 #if WIN32|WINNT
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "UNIX-WIN", "Creating Unix domain socket ('%s') on Windows is not supported.", hostname);
+		XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s', Unix domain socket not supported.\n", hostname);
 		return SOCK_ERR;
 #else
 		return xdebug_create_socket_unix(hostname + strlen("unix://"));
@@ -186,9 +174,9 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 	/* Call getaddrinfo and return SOCK_ERR if the call fails for some reason */
 	if ((status = getaddrinfo(hostname, sport, &hints, &remote)) != 0) {
 #if WIN32|WINNT
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK1", "Creating socket for '%s:%d', getaddrinfo: %d.", hostname, dport, WSAGetLastError());
+		XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', getaddrinfo: %d.\n", hostname, dport, WSAGetLastError());
 #else
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK1", "Creating socket for '%s:%d', getaddrinfo: %s.", hostname, dport, strerror(errno));
+		XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', getaddrinfo: %s.\n", hostname, dport, strerror(errno));
 #endif
 		return SOCK_ERR;
 	}
@@ -199,9 +187,9 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 		 * next IP address in the list */
 		if ((sockfd = socket(ptr->ai_family, ptr->ai_socktype, ptr->ai_protocol)) == SOCK_ERR) {
 #if WIN32|WINNT
-			xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK2", "Creating socket for '%s:%d', socket: %d.", hostname, dport, WSAGetLastError());
+			XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', socket: %d.\n", hostname, dport, WSAGetLastError());
 #else
-			xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK2", "Creating socket for '%s:%d', socket: %s.", hostname, dport, strerror(errno));
+			XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', socket: %s.\n", hostname, dport, strerror(errno));
 #endif
 			continue;
 		}
@@ -210,7 +198,7 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 #ifdef WIN32
 		status = ioctlsocket(sockfd, FIONBIO, &yes);
 		if (SOCKET_ERROR == status) {
-			xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK2", "Creating socket for '%s:%d', FIONBIO: %d.", hostname, dport, WSAGetLastError());
+			XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', FIONBIO: %d.\n", hostname, dport, WSAGetLastError());
 		}
 #else
 		fcntl(sockfd, F_SETFL, O_NONBLOCK);
@@ -219,7 +207,7 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 #if !WIN32 && !WINNT
 		/* Prevent the socket from being inherited by exec'd children on *nix (not necessary on Win) */
 		if (fcntl(sockfd, F_SETFD, FD_CLOEXEC) < 0) {
-			xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK2", "Creating socket for '%s:%d', fcntl(FD_CLOEXEC): %s.", hostname, dport, strerror(errno));
+			XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', fcntl(FD_CLOEXEC): %s.\n", hostname, dport, strerror(errno));
 		}
 #endif
 
@@ -233,17 +221,17 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 #ifdef WIN32
 			errno = WSAGetLastError();
 			if (errno != WSAEINPROGRESS && errno != WSAEWOULDBLOCK) {
-				xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK3", "Creating socket for '%s:%d', connect: %d.", hostname, dport, errno);
+				XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', connect: %d.\n", hostname, dport, errno);
 #else
 			if (errno == EACCES) {
-				xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK3", "Creating socket for '%s:%d', connect: %s.", hostname, dport, strerror(errno));
+				XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', connect: %s.\n", hostname, dport, strerror(errno));
 				SCLOSE(sockfd);
 				sockfd = SOCK_ACCESS_ERR;
 
 				continue;
 			}
 			if (errno != EINPROGRESS) {
-				xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK3", "Creating socket for '%s:%d', connect: %s.", hostname, dport, strerror(errno));
+				XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', connect: %s.\n", hostname, dport, strerror(errno));
 #endif
 				SCLOSE(sockfd);
 				sockfd = SOCK_ERR;
@@ -270,9 +258,9 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 				/* If an error occured when doing the poll */
 				if (sockerror == SOCK_ERR) {
 #if WIN32|WINNT
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK4", "Creating socket for '%s:%d', WSAPoll error: %d (%d, %d).", hostname, dport, WSAGetLastError(), sockerror, errno);
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', WSAPoll error: %d (%d, %d).\n", hostname, dport, WSAGetLastError(), sockerror, errno);
 #else
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK4", "Creating socket for '%s:%d', poll error: %s (%d).", hostname, dport, strerror(errno), sockerror);
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', poll error: %s (%d).\n", hostname, dport, strerror(errno), sockerror);
 #endif
 					sockerror = SOCK_ERR;
 					break;
@@ -287,9 +275,9 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 				/* If the poll was successful but an error occured */
 				if (ufds[0].revents & (POLLERR | POLLHUP | POLLNVAL)) {
 #if WIN32|WINNT
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK4", "Creating socket for '%s:%d', WSAPoll success, but error: %d (%d).", hostname, dport, WSAGetLastError(), ufds[0].revents);
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', WSAPoll success, but error: %d (%d).\n", hostname, dport, WSAGetLastError(), ufds[0].revents);
 #else
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK4", "Creating socket for '%s:%d', poll success, but error: %s (%d).", hostname, dport, strerror(errno), ufds[0].revents);
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', poll success, but error: %s (%d).\n", hostname, dport, strerror(errno), ufds[0].revents);
 #endif
 					sockerror = SOCK_ERR;
 					break;
@@ -302,9 +290,9 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 				} else {
 					/* We should never get here, but added as a failsafe to break out from any loops */
 #if WIN32|WINNT
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK4", "Creating socket for '%s:%d', WSAPoll: %d.", hostname, dport, WSAGetLastError());
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', WSAPoll: %d.\n", hostname, dport, WSAGetLastError());
 #else
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK4", "Creating socket for '%s:%d', poll: %s.", hostname, dport, strerror(errno));
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', poll: %s.\n", hostname, dport, strerror(errno));
 #endif
 					sockerror = SOCK_ERR;
 					break;
@@ -315,9 +303,9 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 				actually_connected = getpeername(sockfd, (struct sockaddr *)&sa, &size);
 				if (actually_connected == -1) {
 #if WIN32|WINNT
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK5", "Creating socket for '%s:%d', getpeername: %d.", hostname, dport, WSAGetLastError());
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', getpeername: %d.\n", hostname, dport, WSAGetLastError());
 #else
-					xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK5", "Creating socket for '%s:%d', getpeername: %s.", hostname, dport, strerror(errno));
+					XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', getpeername: %s.\n", hostname, dport, strerror(errno));
 #endif
 					sockerror = SOCK_ERR;
 				}
@@ -343,7 +331,7 @@ static int xdebug_create_socket(const char *hostname, int dport, int timeout)
 #ifdef WIN32
 		status = ioctlsocket(sockfd, FIONBIO, &no);
 		if (SOCKET_ERROR == status) {
-			xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "SOCK6", "Creating socket for '%s:%d', FIONBIO: %d.", hostname, dport, WSAGetLastError());
+			XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Creating socket for '%s:%d', FIONBIO: %d.\n", hostname, dport, WSAGetLastError());
 		}
 #else
 		fcntl(sockfd, F_SETFL, 0);
@@ -363,132 +351,125 @@ void xdebug_close_socket(int socketfd)
 	SCLOSE(socketfd);
 }
 
+/* Log related functions */
+static void xdebug_open_log(void)
+{
+	zend_ulong pid = xdebug_get_pid();
+
+	/* initialize remote log file */
+	XG_DBG(remote_log_file) = NULL;
+	if (XINI_DBG(remote_log) && strlen(XINI_DBG(remote_log))) {
+		XG_DBG(remote_log_file) = xdebug_fopen(XINI_DBG(remote_log), "a", NULL, NULL);
+	}
+	if (XG_DBG(remote_log_file)) {
+		char *timestr = xdebug_get_time();
+		fprintf(XG_DBG(remote_log_file), "[" ZEND_ULONG_FMT "] Log opened at %s\n", pid, timestr);
+		fflush(XG_DBG(remote_log_file));
+		xdfree(timestr);
+	} else if (strlen(XINI_DBG(remote_log))) {
+		php_log_err(xdebug_sprintf("Xdebug could not open the remote debug file '%s'.", XINI_DBG(remote_log)));
+	}
+}
+
+static void xdebug_close_log()
+{
+
+	if (XG_DBG(remote_log_file)) {
+		zend_ulong pid = xdebug_get_pid();
+		char *timestr = xdebug_get_time();
+
+		fprintf(XG_DBG(remote_log_file), "[" ZEND_ULONG_FMT "] Log closed at %s\n\n", pid, timestr);
+		fflush(XG_DBG(remote_log_file));
+		xdfree(timestr);
+		fclose(XG_DBG(remote_log_file));
+		XG_DBG(remote_log_file) = NULL;
+	}
+}
+
 /* Starting the debugger */
-static void xdebug_init_normal_debugger(xdebug_str *connection_attempts)
-{
-	zval *remote_addr = NULL;
-	char *cp = NULL;
-	int   cp_found = 0;
-	const char *header = NULL;
-
-	if (!XINI_DBG(discover_client_host)) {
-		xdebug_str_add_fmt(connection_attempts, "%s:%ld (through xdebug.client_host/xdebug.client_port)", XINI_DBG(client_host), XINI_DBG(client_port));
-		xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Connecting to configured address/port: %s:%ld.", XINI_DBG(client_host), (long int) XINI_DBG(client_port));
-
-		XG_DBG(context).socket = xdebug_create_socket(XINI_DBG(client_host), XINI_DBG(client_port), XINI_DBG(connect_timeout_ms));
-		return;
-	}
-
-	xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Checking remote connect back address.");
-	if (XINI_DBG(client_discovery_header) && XINI_DBG(client_discovery_header)[0]) {
-		header = XINI_DBG(client_discovery_header);
-		xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Checking user configured header '%s'.", XINI_DBG(client_discovery_header));
-
-		remote_addr = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), XINI_DBG(client_discovery_header), HASH_KEY_STRLEN(XINI_DBG(client_discovery_header)));
-	}
-	if (!remote_addr) {
-		header = "HTTP_X_FORWARDED_FOR";
-		xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Checking header 'HTTP_X_FORWARDED_FOR'.");
-
-		remote_addr = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), "HTTP_X_FORWARDED_FOR", HASH_KEY_SIZEOF("HTTP_X_FORWARDED_FOR"));
-	}
-	if (!remote_addr) {
-		header = "REMOTE_ADDR";
-		xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Checking header 'REMOTE_ADDR'.");
-
-		remote_addr = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), "REMOTE_ADDR", HASH_KEY_SIZEOF("REMOTE_ADDR"));
-	}
-
-	if (remote_addr && strstr(Z_STRVAL_P(remote_addr), "://")) {
-		header = NULL;
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "INVADDR", "Invalid remote address provided containing URI spec '%s'.", Z_STRVAL_P(remote_addr));
-
-		remote_addr = NULL;
-	}
-
-	if (!remote_addr) {
-		xdebug_str_add_fmt(connection_attempts, "%s:%ld (fallback through xdebug.client_host/xdebug.client_port)", XINI_DBG(client_host), XINI_DBG(client_port));
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "HDR", "Could not discover client host through HTTP headers, connecting to configured address/port: %s:%ld. :-|", XINI_DBG(client_host), (long int) XINI_DBG(client_port));
-
-		XG_DBG(context).socket = xdebug_create_socket(XINI_DBG(client_host), XINI_DBG(client_port), XINI_DBG(connect_timeout_ms));
-		return;
-	}
-
-	/* Use first IP according to RFC 7239 */
-	cp = strchr(Z_STRVAL_P(remote_addr), ',');
-	if (cp) {
-		*cp = '\0';
-		cp_found = 1;
-	}
-
-	xdebug_str_add_fmt(connection_attempts, "%s:%ld (from %s HTTP header)", Z_STRVAL_P(remote_addr), XINI_DBG(client_port), header);
-	xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Client host discovered through HTTP header, connecting to %s:%ld.", Z_STRVAL_P(remote_addr), (long int) XINI_DBG(client_port));
-
-	XG_DBG(context).socket = xdebug_create_socket(Z_STRVAL_P(remote_addr), XINI_DBG(client_port), XINI_DBG(connect_timeout_ms));
-
-	if (XG_DBG(context).socket < 0) {
-		xdebug_str_add_fmt(connection_attempts, ", %s:%ld (fallback through xdebug.client_host/xdebug.client_port)", XINI_DBG(client_host), XINI_DBG(client_port));
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_WARN, "CON", "Could not connect to client host discovered through HTTP headers, connecting to configured address/port: %s:%ld. :-|", XINI_DBG(client_host), (long int) XINI_DBG(client_port));
-
-		XG_DBG(context).socket = xdebug_create_socket(XINI_DBG(client_host), XINI_DBG(client_port), XINI_DBG(connect_timeout_ms));
-	}
-
-	/* Replace the ',', in case we had changed the original header due
-	 * to multiple values */
-	if (cp_found) {
-		*cp = ',';
-	}
-}
-
-static void xdebug_init_cloud_debugger()
-{
-	unsigned long  crc = xdebug_crc32(XINI_DBG(cloud_id), strlen(XINI_DBG(cloud_id)));
-	char          *host;
-
-	host = xdebug_sprintf("%c.cloud.xdebug.com", (crc & 0x0f) + 'a');
-
-	xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Connecting to configured address/port: %s:%ld.", host, 9020L);
-	XG_DBG(context).socket = xdebug_create_socket(host, 9020, XINI_DBG(connect_timeout_ms));
-
-	xdfree(host);
-}
-
 static void xdebug_init_debugger()
 {
-	xdebug_str *connection_attempts = xdebug_str_new();
+	xdebug_open_log();
 
 	/* Get handler from mode */
 	XG_DBG(context).handler = &xdebug_handler_dbgp;
 
-	if (strcmp(XINI_DBG(cloud_id), "") == 0) {
-		xdebug_init_normal_debugger(connection_attempts);
-	} else {
-		xdebug_init_cloud_debugger();
-	}
+	if (XINI_DBG(remote_connect_back)) {
+		zval *remote_addr = NULL;
 
-	/* Check whether we're connected, or why not */
+		XG_DBG(context).handler->log(XDEBUG_LOG_INFO, "Checking remote connect back address.\n");
+		if (XINI_DBG(remote_addr_header) && XINI_DBG(remote_addr_header)[0]) {
+			XG_DBG(context).handler->log(XDEBUG_LOG_INFO, "Checking user configured header '%s'.\n", XINI_DBG(remote_addr_header));
+			remote_addr = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), XINI_DBG(remote_addr_header), HASH_KEY_STRLEN(XINI_DBG(remote_addr_header)));
+		}
+		if (!remote_addr) {
+			XG_DBG(context).handler->log(XDEBUG_LOG_INFO, "Checking header 'HTTP_X_FORWARDED_FOR'.\n");
+			remote_addr = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), "HTTP_X_FORWARDED_FOR", HASH_KEY_SIZEOF("HTTP_X_FORWARDED_FOR"));
+		}
+		if (!remote_addr) {
+			XG_DBG(context).handler->log(XDEBUG_LOG_INFO, "Checking header 'REMOTE_ADDR'.\n");
+			remote_addr = zend_hash_str_find(Z_ARRVAL(PG(http_globals)[TRACK_VARS_SERVER]), "REMOTE_ADDR", HASH_KEY_SIZEOF("REMOTE_ADDR"));
+		}
+
+		if (remote_addr && strstr(Z_STRVAL_P(remote_addr), "://")) {
+			XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Invalid remote address provided containing URI spec '%s'.\n", Z_STRVAL_P(remote_addr));
+			remote_addr = NULL;
+		}
+
+		if (remote_addr) {
+			char *cp = NULL;
+			int   cp_found = 0;
+
+			/* Use first IP according to RFC 7239 */
+			cp = strchr(Z_STRVAL_P(remote_addr), ',');
+			if (cp) {
+				*cp = '\0';
+				cp_found = 1;
+			}
+
+			XG_DBG(context).handler->log(XDEBUG_LOG_INFO, "Remote address found, connecting to %s:%ld.\n", Z_STRVAL_P(remote_addr), (long int) XINI_DBG(remote_port));
+			XG_DBG(context).socket = xdebug_create_socket(Z_STRVAL_P(remote_addr), XINI_DBG(remote_port), XINI_DBG(remote_connect_timeout));
+
+			/* Replace the ',', in case we had changed the original header due
+			 * to multiple values */
+			if (cp_found) {
+				*cp = ',';
+			}
+		} else {
+			XG_DBG(context).handler->log(XDEBUG_LOG_WARN, "Remote address not found, connecting to configured address/port: %s:%ld. :-|\n", XINI_DBG(remote_host), (long int) XINI_DBG(remote_port));
+			XG_DBG(context).socket = xdebug_create_socket(XINI_DBG(remote_host), XINI_DBG(remote_port), XINI_DBG(remote_connect_timeout));
+		}
+	} else {
+		XG_DBG(context).handler->log(XDEBUG_LOG_INFO, "Connecting to configured address/port: %s:%ld.\n", XINI_DBG(remote_host), (long int) XINI_DBG(remote_port));
+		XG_DBG(context).socket = xdebug_create_socket(XINI_DBG(remote_host), XINI_DBG(remote_port), XINI_DBG(remote_connect_timeout));
+	}
 	if (XG_DBG(context).socket >= 0) {
-		xdebug_log(XLOG_CHAN_DEBUG, XLOG_INFO, "Connected to debugging client: %s. :-)", connection_attempts->d);
+		XG_DBG(context).handler->log(XDEBUG_LOG_INFO, "Connected to client. :-)\n");
 		xdebug_mark_debug_connection_pending();
 
 		if (!XG_DBG(context).handler->remote_init(&(XG_DBG(context)), XDEBUG_REQ)) {
 			/* The request could not be started, ignore it then */
-			xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_ERR, "SES-INIT", "The debug session could not be started. Tried: %s. :-(", connection_attempts->d);
+			XG_DBG(context).handler->log(XDEBUG_LOG_ERR, "The debug session could not be started. :-(\n");
 		} else {
 			/* All is well, turn off script time outs */
-			zend_unset_timeout();
-			EG(timeout_seconds) = 0;
-			zend_set_timeout(EG(timeout_seconds), 0);
+			zend_string *ini_name = zend_string_init("max_execution_time", sizeof("max_execution_time") - 1, 0);
+			zend_string *ini_val = zend_string_init("0", sizeof("0") - 1, 0);
+
+			zend_alter_ini_entry(ini_name, ini_val, PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
+
+			zend_string_release(ini_val);
+			zend_string_release(ini_name);
 		}
 	} else if (XG_DBG(context).socket == -1) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_ERR, "NOCON", "Could not connect to debugging client. Tried: %s :-(", connection_attempts->d);
+		XG_DBG(context).handler->log(XDEBUG_LOG_ERR, "Could not connect to client. :-(\n");
 	} else if (XG_DBG(context).socket == -2) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_ERR, "TIMEOUT", "Time-out connecting to debugging client, waited: " ZEND_LONG_FMT " ms. Tried: %s :-(", XINI_DBG(connect_timeout_ms), connection_attempts->d);
+		XG_DBG(context).handler->log(XDEBUG_LOG_ERR, "Time-out connecting to client (Waited: " ZEND_LONG_FMT " ms). :-(\n", XINI_DBG(remote_connect_timeout));
 	} else if (XG_DBG(context).socket == -3) {
-		xdebug_log_ex(XLOG_CHAN_DEBUG, XLOG_ERR, "NOPERM", "No permission connecting to debugging client (%s). This could be SELinux related. :-(", connection_attempts->d);
+		XG_DBG(context).handler->log(XDEBUG_LOG_ERR, "No permission connecting to client. This could be SELinux related. :-(\n");
+	}
+	if (!XG_DBG(remote_connection_enabled)) {
+		xdebug_close_log();
 	}
-
-	xdebug_str_free(connection_attempts);
 }
 
 void xdebug_abort_debugger()
@@ -507,7 +488,31 @@ void xdebug_restart_debugger()
 /* Remote connection activation and house keeping */
 int xdebug_is_debug_connection_active()
 {
-	return XG_DBG(remote_connection_enabled);
+	return (
+		XG_DBG(remote_connection_enabled)
+	);
+}
+
+int xdebug_is_debug_connection_active_for_current_pid()
+{
+	zend_ulong pid;
+
+	/* Early return to save some getpid() calls */
+	if (!xdebug_is_debug_connection_active()) {
+		return 0;
+	}
+
+	pid = xdebug_get_pid();
+
+	/* Start debugger if previously a connection was established and this
+	 * process no longer has the same PID */
+	if (XG_DBG(remote_connection_pid) != pid) {
+		xdebug_restart_debugger();
+	}
+
+	return (
+		XG_DBG(remote_connection_enabled) && (XG_DBG(remote_connection_pid) == pid)
+	);
 }
 
 void xdebug_mark_debug_connection_active()
@@ -526,34 +531,20 @@ void xdebug_mark_debug_connection_not_active()
 {
 	if (XG_DBG(remote_connection_enabled)) {
 		xdebug_close_socket(XG_DBG(context).socket);
+		xdebug_close_log();
 	}
 
 	XG_DBG(remote_connection_enabled) = 0;
 	XG_DBG(remote_connection_pid) = 0;
 }
 
-void xdebug_debug_init_if_requested_on_error()
+void xdebug_do_jit()
 {
-	RETURN_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
-
-	if (!xdebug_lib_start_upon_error()) {
-		return;
-	}
-
-	if (!xdebug_is_debug_connection_active()) {
-		xdebug_init_debugger();
-	}
-}
-
-void xdebug_debug_init_if_requested_on_xdebug_break()
-{
-	RETURN_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
-
-	if (xdebug_is_debug_connection_active()) {
-		return;
-	}
-
-	if (xdebug_lib_start_if_mode_is_trigger(XDEBUG_MODE_STEP_DEBUG)) {
+	if (
+		(XINI_DBG(remote_mode) == XDEBUG_JIT) &&
+		!xdebug_is_debug_connection_active_for_current_pid() &&
+		XINI_DBG(remote_enable)
+	) {
 		xdebug_init_debugger();
 	}
 }
@@ -570,13 +561,10 @@ static int xdebug_handle_start_session()
 {
 	int   activate_session = 0;
 	zval *dummy;
-	char *dummy_env;
 
 	/* Set session cookie if requested */
 	if (
 		((
-			(dummy = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_ENV]), "XDEBUG_SESSION_START", sizeof("XDEBUG_SESSION_START") - 1)) != NULL
-		) || (
 			(dummy = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_GET]), "XDEBUG_SESSION_START", sizeof("XDEBUG_SESSION_START") - 1)) != NULL
 		) || (
 			(dummy = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_POST]), "XDEBUG_SESSION_START", sizeof("XDEBUG_SESSION_START") - 1)) != NULL
@@ -586,17 +574,7 @@ static int xdebug_handle_start_session()
 		convert_to_string_ex(dummy);
 		xdebug_update_ide_key(Z_STRVAL_P(dummy));
 
-		xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1, Z_STRVAL_P(dummy), Z_STRLEN_P(dummy), time(NULL) + XDEBUG_COOKIE_EXPIRE_TIME, "/", 1, NULL, 0, 0, 1, 0);
-		activate_session = 1;
-	} else if (
-		(dummy_env = getenv("XDEBUG_SESSION_START")) != NULL
-	) {
-		xdebug_update_ide_key(dummy_env);
-
-		if (!SG(headers_sent)) {
-			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1, XG_DBG(ide_key), strlen(XG_DBG(ide_key)), time(NULL) + XDEBUG_COOKIE_EXPIRE_TIME, "/", 1, NULL, 0, 0, 1, 0);
-		}
-
+		xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1, Z_STRVAL_P(dummy), Z_STRLEN_P(dummy), time(NULL) + XINI_DBG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0);
 		activate_session = 1;
 	} else if (
 		(dummy = zend_hash_str_find(Z_ARR(PG(http_globals)[TRACK_VARS_COOKIE]), "XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1)) != NULL
@@ -607,9 +585,9 @@ static int xdebug_handle_start_session()
 		activate_session = 1;
 	} else if (getenv("XDEBUG_CONFIG")) {
 		if (XG_DBG(ide_key) && *XG_DBG(ide_key) && !SG(headers_sent)) {
-			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1, XG_DBG(ide_key), strlen(XG_DBG(ide_key)), time(NULL) + XDEBUG_COOKIE_EXPIRE_TIME, "/", 1, NULL, 0, 0, 1, 0);
-			activate_session = 1;
+			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1, XG_DBG(ide_key), strlen(XG_DBG(ide_key)), time(NULL) + XINI_DBG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0);
 		}
+		activate_session = 1;
 	}
 
 	return activate_session;
@@ -626,36 +604,27 @@ static void xdebug_handle_stop_session()
 		))
 		&& !SG(headers_sent)
 	) {
-		xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1, (char*) "", 0, time(NULL) + XDEBUG_COOKIE_EXPIRE_TIME, "/", 1, NULL, 0, 0, 1, 0);
+		xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION") - 1, (char*) "", 0, time(NULL) + XINI_DBG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0);
 	}
 }
 
-void xdebug_debug_init_if_requested_at_startup(void)
+void xdebug_do_req(void)
 {
-	char *found_trigger_value = NULL;
-
 	if (XG_DBG(detached)) {
 		return;
 	}
 
-	if (xdebug_is_debug_connection_active()) {
+	if (XINI_DBG(remote_mode) != XDEBUG_REQ) {
 		return;
 	}
 
 	if (
-		xdebug_lib_start_with_request() ||
-		(!xdebug_lib_never_start_with_request() && xdebug_handle_start_session()) ||
-		xdebug_lib_start_with_trigger(XDEBUG_MODE_STEP_DEBUG, &found_trigger_value)
+		XINI_DBG(remote_enable) &&
+		!xdebug_is_debug_connection_active_for_current_pid() &&
+		(XINI_DBG(remote_autostart) || xdebug_handle_start_session())
 	) {
-		if (found_trigger_value) {
-			xdebug_update_ide_key(found_trigger_value);
-		}
 		xdebug_init_debugger();
 	}
 
-	if (found_trigger_value) {
-		xdfree(found_trigger_value);
-	}
-
 	xdebug_handle_stop_session();
 }
