@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2021 Derick Rethans                               |
+   | Copyright (c) 2002-2020 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,6 +12,8 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
+   | Authors: Derick Rethans <derick@xdebug.org>                          |
+   +----------------------------------------------------------------------+
  */
 
 #include "php_xdebug.h"
@@ -20,7 +22,7 @@
 #include "zend_exceptions.h"
 
 #include "debugger_private.h"
-#include "lib/var.h"
+#include "base/stack.h"
 
 extern ZEND_DECLARE_MODULE_GLOBALS(xdebug);
 
@@ -29,6 +31,23 @@ static size_t xdebug_ub_write(const char *string, size_t length);
 
 static void xdebug_line_list_dtor(xdebug_lines_list *line_list);
 
+PHP_INI_MH(OnUpdateDebugMode)
+{
+	if (!new_value) {
+		XINI_DBG(remote_mode) = XDEBUG_NONE;
+
+	} else if (strcmp(STR_NAME_VAL(new_value), "jit") == 0) {
+		XINI_DBG(remote_mode) = XDEBUG_JIT;
+
+	} else if (strcmp(STR_NAME_VAL(new_value), "req") == 0) {
+		XINI_DBG(remote_mode) = XDEBUG_REQ;
+
+	} else {
+		XINI_DBG(remote_mode) = XDEBUG_NONE;
+	}
+	return SUCCESS;
+}
+
 void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 {
 	xg->breakpoint_count     = 0;
@@ -36,17 +55,16 @@ void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 	xg->stdout_mode          = 0;
 	xg->no_exec              = 0;
 	xg->context.program_name = NULL;
-	xg->context.list.last_filename = NULL;
+	xg->context.list.last_file = NULL;
 	xg->context.list.last_line = 0;
 	xg->context.do_break     = 0;
-	xg->context.pending_breakpoint = NULL;
 	xg->context.do_step      = 0;
 	xg->context.do_next      = 0;
 	xg->context.do_finish    = 0;
-	xg->context.do_connect_to_client = 0;
 
 	xg->remote_connection_enabled = 0;
 	xg->remote_connection_pid     = 0;
+	xg->remote_log_file           = 0;
 	xg->breakpoints_allowed       = 0;
 
 	/* Capturing output */
@@ -54,11 +72,6 @@ void xdebug_init_debugger_globals(xdebug_debugger_globals_t *xg)
 		xdebug_orig_ub_write = sapi_module.ub_write;
 		sapi_module.ub_write = xdebug_ub_write;
 	}
-
-	/* Statistics and diagnostics */
-	xg->context.connected_hostname = NULL;
-	xg->context.connected_port = 0;
-	xg->context.detached_message = NULL;
 }
 
 static char *xdebug_debugger_get_ide_key(void)
@@ -75,6 +88,16 @@ static char *xdebug_debugger_get_ide_key(void)
 		return ide_key;
 	}
 
+	ide_key = getenv("USER");
+	if (ide_key && *ide_key) {
+		return ide_key;
+	}
+
+	ide_key = getenv("USERNAME");
+	if (ide_key && *ide_key) {
+		return ide_key;
+	}
+
 	return NULL;
 }
 
@@ -96,29 +119,10 @@ int xdebug_debugger_bailout_if_no_exec_requested(void)
 	return 0;
 }
 
-static void register_compiled_variables(void)
-{
-	function_stack_entry *loop_fse = XDEBUG_VECTOR_TAIL(XG_BASE(stack));
-	int                   i;
-
-	for (i = 0; i < XDEBUG_VECTOR_COUNT(XG_BASE(stack)); i++, loop_fse--) {
-		if (loop_fse->declared_vars) {
-			continue;
-		}
-
-		if (loop_fse->user_defined == XDEBUG_BUILT_IN) {
-			continue;
-		}
-
-		xdebug_lib_register_compiled_variables(loop_fse, loop_fse->op_array);
-	}
-}
-
-
 void xdebug_debugger_set_program_name(zend_string *filename)
 {
 	if (!XG_DBG(context).program_name) {
-		XG_DBG(context).program_name = zend_string_copy(filename);
+		XG_DBG(context).program_name = xdstrdup(STR_NAME_VAL(filename));
 	}
 }
 
@@ -161,67 +165,8 @@ static int xdebug_handle_hit_value(xdebug_brk_info *brk_info)
 	return 0;
 }
 
-int xdebug_do_eval(char *eval_string, zval *ret_zval)
-{
-#if PHP_VERSION_ID < 80000
-	int                old_track_errors;
-#endif
-	int                res = 1;
-	zend_execute_data *original_execute_data = EG(current_execute_data);
-	int                original_no_extensions = EG(no_extensions);
-	zend_object       *original_exception = EG(exception);
-	JMP_BUF           *original_bailout = EG(bailout);
 
-	/* Remember error reporting level and track errors */
-	XG_BASE(error_reporting_override) = EG(error_reporting);
-	XG_BASE(error_reporting_overridden) = 1;
-#if PHP_VERSION_ID < 80000
-	old_track_errors = PG(track_errors);
-#endif
-	EG(error_reporting) = 0;
-#if PHP_VERSION_ID < 80000
-	PG(track_errors) = 0;
-#endif
-
-	XG_DBG(context).inhibit_notifications = 1;
-	XG_DBG(breakpoints_allowed) = 0;
-
-	/* Reset exception in case we're triggered while being in xdebug_throw_exception_hook */
-	EG(exception) = NULL;
-
-	/* Do evaluation */
-	zend_first_try {
-		res = (zend_eval_string(eval_string, ret_zval, (char*) "xdebug://debug-eval") == SUCCESS);
-	} zend_end_try();
-
-	/* FIXME: Bubble up exception message to DBGp return packet */
-	if (EG(exception)) {
-#if PHP_VERSION_ID >= 80000
-		if (!res) {
-			zend_clear_exception();
-		}
-#endif
-		res = 0;
-	}
-
-	/* Clean up */
-	EG(error_reporting) = XG_BASE(error_reporting_override);
-	XG_BASE(error_reporting_overridden) = 0;
-#if PHP_VERSION_ID < 80000
-	PG(track_errors) = old_track_errors;
-#endif
-	XG_DBG(breakpoints_allowed) = 1;
-	XG_DBG(context).inhibit_notifications = 0;
-
-	EG(current_execute_data) = original_execute_data;
-	EG(no_extensions) = original_no_extensions;
-	EG(exception) = original_exception;
-	EG(bailout) = original_bailout;
-
-	return res;
-}
-
-void xdebug_debugger_statement_call(zend_string *filename, int lineno)
+void xdebug_debugger_statement_call(char *file, int file_len, int lineno)
 {
 	xdebug_llist_element *le;
 	xdebug_brk_info      *extra_brk_info;
@@ -229,31 +174,21 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 	int                   level = 0;
 	int                   func_nr = 0;
 
-	if (XG_DBG(context).do_connect_to_client) {
-		XG_DBG(context).do_connect_to_client = 0;
+	if (xdebug_is_debug_connection_active_for_current_pid()) {
 
-		if (!xdebug_is_debug_connection_active()) {
-			xdebug_debug_init_if_requested_on_xdebug_break();
-			register_compiled_variables();
-		}
-	}
-
-	if (xdebug_is_debug_connection_active()) {
 		if (XG_DBG(context).do_break) {
-			xdebug_brk_info *brk_info = XG_DBG(context).pending_breakpoint;
-
 			XG_DBG(context).do_break = 0;
-			XG_DBG(context).pending_breakpoint = NULL;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL, brk_info)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
 		}
 
 		/* Get latest stack level and function number */
-		if (XG_BASE(stack) && XDEBUG_VECTOR_TAIL(XG_BASE(stack))) {
-			fse = XDEBUG_VECTOR_TAIL(XG_BASE(stack));
+		if (XG_BASE(stack) && XDEBUG_LLIST_TAIL(XG_BASE(stack))) {
+			le = XDEBUG_LLIST_TAIL(XG_BASE(stack));
+			fse = XDEBUG_LLIST_VALP(le);
 			level = fse->level;
 			func_nr = fse->function_nr;
 		} else {
@@ -271,7 +206,7 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 		) {
 			XG_DBG(context).do_finish = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -282,7 +217,7 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 		if (XG_DBG(context).do_next && XG_DBG(context).next_level >= level) {
 			XG_DBG(context).do_next = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -293,7 +228,7 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 		if (XG_DBG(context).do_step) {
 			XG_DBG(context).do_step = 0;
 
-			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_STEP, NULL, 0, NULL, NULL)) {
+			if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_STEP, NULL, 0, NULL)) {
 				xdebug_mark_debug_connection_not_active();
 				return;
 			}
@@ -301,13 +236,13 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 		}
 
 		if (XG_DBG(context).line_breakpoints) {
-			int   break_ok, res;
+			int   break_ok;
 			zval  retval;
 
 			for (le = XDEBUG_LLIST_HEAD(XG_DBG(context).line_breakpoints); le != NULL; le = XDEBUG_LLIST_NEXT(le)) {
 				extra_brk_info = XDEBUG_LLIST_VALP(le);
 
-				if (XG_DBG(context).handler->break_on_line(&(XG_DBG(context)), extra_brk_info, filename, lineno)) {
+				if (XG_DBG(context).handler->break_on_line(&(XG_DBG(context)), extra_brk_info, file, file_len, lineno)) {
 					break_ok = 1; /* Breaking is allowed by default */
 
 					/* Check if we have a condition set for it */
@@ -318,14 +253,24 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 						break_ok = 0;
 
 						/* Remember error reporting level */
-						res = xdebug_do_eval(extra_brk_info->condition, &retval);
-						if (res) {
+						XG_BASE(error_reporting_override) = EG(error_reporting);
+						XG_BASE(error_reporting_overridden) = 1;
+						EG(error_reporting) = 0;
+						XG_DBG(context).inhibit_notifications = 1;
+
+						/* Check the condition */
+						if (zend_eval_string(extra_brk_info->condition, &retval, (char*) "xdebug conditional breakpoint") == SUCCESS) {
 							break_ok = Z_TYPE(retval) == IS_TRUE;
 							zval_dtor(&retval);
 						}
+
+						/* Restore error reporting level */
+						EG(error_reporting) = XG_BASE(error_reporting_override);
+						XG_BASE(error_reporting_overridden) = 0;
+						XG_DBG(context).inhibit_notifications = 0;
 					}
 					if (break_ok && xdebug_handle_hit_value(extra_brk_info)) {
-						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), filename, lineno, XDEBUG_BREAK, NULL, 0, NULL, extra_brk_info)) {
+						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), file, lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
 							xdebug_mark_debug_connection_not_active();
 							break;
 						}
@@ -337,21 +282,14 @@ void xdebug_debugger_statement_call(zend_string *filename, int lineno)
 	}
 }
 
-#if PHP_VERSION_ID >= 80000
-void xdebug_debugger_throw_exception_hook(zend_object *exception, zval *file, zval *line, zval *code, char *code_str, zval *message)
+void xdebug_debugger_throw_exception_hook(zend_class_entry * exception_ce, zval *file, zval *line, zval *code, char *code_str, zval *message)
 {
-	zend_class_entry *exception_ce = exception->ce;
-#else
-void xdebug_debugger_throw_exception_hook(zval *exception, zval *file, zval *line, zval *code, char *code_str, zval *message)
-{
-	zend_class_entry *exception_ce = Z_OBJCE_P(exception);
-#endif
 	xdebug_brk_info *extra_brk_info;
 
 	/* Start JIT if requested and not yet enabled */
-	xdebug_debug_init_if_requested_on_error();
+	xdebug_do_jit();
 
-	if (xdebug_is_debug_connection_active() && XG_DBG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(breakpoints_allowed)) {
 		int exception_breakpoint_found = 0;
 
 		/* Check if we have a wild card exception breakpoint */
@@ -375,15 +313,12 @@ void xdebug_debugger_throw_exception_hook(zval *exception, zval *file, zval *lin
 		}
 #endif
 		if (exception_breakpoint_found && xdebug_handle_hit_value(extra_brk_info)) {
-			if (
-				!XG_DBG(context).handler->remote_breakpoint(
-					&(XG_DBG(context)), XG_BASE(stack),
-					Z_STR_P(file), Z_LVAL_P(line), XDEBUG_BREAK,
-					(char*) STR_NAME_VAL(exception_ce->name),
-					code_str ? code_str : ((code && Z_TYPE_P(code) == IS_STRING) ? Z_STRVAL_P(code) : NULL),
-					message ? Z_STRVAL_P(message) : "",
-					extra_brk_info
-				)
+			if (!XG_DBG(context).handler->remote_breakpoint(
+				&(XG_DBG(context)), XG_BASE(stack),
+				Z_STRVAL_P(file), Z_LVAL_P(line), XDEBUG_BREAK,
+				(char*) STR_NAME_VAL(exception_ce->name),
+				code_str ? code_str : ((code && Z_TYPE_P(code) == IS_STRING) ? Z_STRVAL_P(code) : NULL),
+				Z_STRVAL_P(message))
 			) {
 				xdebug_mark_debug_connection_not_active();
 			}
@@ -391,14 +326,14 @@ void xdebug_debugger_throw_exception_hook(zval *exception, zval *file, zval *lin
 	}
 }
 
-void xdebug_debugger_error_cb(zend_string *error_filename, int error_lineno, int type, char *error_type_str, char *buffer)
+void xdebug_debugger_error_cb(const char *error_filename, int error_lineno, int type, char *error_type_str, char *buffer)
 {
 	xdebug_brk_info *extra_brk_info = NULL;
 
 	/* Start JIT if requested and not yet enabled */
-	xdebug_debug_init_if_requested_on_error();
+	xdebug_do_jit();
 
-	if (xdebug_is_debug_connection_active() && XG_DBG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(breakpoints_allowed)) {
 		/* Send notification with warning/notice/error information */
 		if (XG_DBG(context).send_notifications && !XG_DBG(context).inhibit_notifications) {
 			if (!XG_DBG(context).handler->remote_notification(&(XG_DBG(context)), error_filename, error_lineno, type, error_type_str, buffer)) {
@@ -414,7 +349,7 @@ void xdebug_debugger_error_cb(zend_string *error_filename, int error_lineno, int
 			if (xdebug_handle_hit_value(extra_brk_info)) {
 				char *type_str = xdebug_sprintf("%ld", type);
 
-				if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), error_filename, error_lineno, XDEBUG_BREAK, error_type_str, type_str, buffer, extra_brk_info)) {
+				if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), (char *) error_filename, error_lineno, XDEBUG_BREAK, error_type_str, type_str, buffer)) {
 					xdebug_mark_debug_connection_not_active();
 				}
 
@@ -438,12 +373,11 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 			if (!extra_brk_info->disabled && (extra_brk_info->function_break_type == breakpoint_type)) {
 				if (xdebug_handle_hit_value(extra_brk_info)) {
 					if (fse->user_defined == XDEBUG_BUILT_IN || (breakpoint_type == XDEBUG_BREAKPOINT_TYPE_RETURN)) {
-						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL, extra_brk_info)) {
+						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
 							return 0;
 						}
 					} else {
 						XG_DBG(context).do_break = 1;
-						XG_DBG(context).pending_breakpoint = extra_brk_info;
 					}
 				}
 			}
@@ -451,13 +385,11 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 	}
 	/* class->function breakpoints */
 	else if (fse->function.type == XFUNC_MEMBER || fse->function.type == XFUNC_STATIC_MEMBER) {
-		/* Using strlen(ZSTR_VAL(...)) here to cut of the string at the first \0, which is needed
-		 * for anonymous classes, in combination with the snprintf() below */
-		tmp_len = strlen(ZSTR_VAL(fse->function.object_class)) + strlen(fse->function.function) + 3;
-		tmp_name = xdmalloc(tmp_len);
 		/* We intentionally do not use xdebug_sprintf because it can create a bottleneck in large
-		 * codebases due to setlocale calls. We don't care about the locale here. */
-		snprintf(tmp_name, tmp_len, "%s::%s", ZSTR_VAL(fse->function.object_class), fse->function.function);
+		   codebases due to setlocale calls. We don't care about the locale here. */
+		tmp_len = strlen(fse->function.class) + strlen(fse->function.function) + 3;
+		tmp_name = xdmalloc(tmp_len);
+		snprintf(tmp_name, tmp_len, "%s::%s", fse->function.class, fse->function.function);
 
 		if (xdebug_hash_find(XG_DBG(context).function_breakpoints, tmp_name, tmp_len - 1, (void *) &extra_brk_info)) {
 			/* Yup, breakpoint found, call handler if the breakpoint is not
@@ -465,12 +397,11 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 			if (!extra_brk_info->disabled && (extra_brk_info->function_break_type == breakpoint_type)) {
 				if (xdebug_handle_hit_value(extra_brk_info)) {
 					if (fse->user_defined == XDEBUG_BUILT_IN || (breakpoint_type == XDEBUG_BREAKPOINT_TYPE_RETURN)) {
-						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL, extra_brk_info)) {
+						if (!XG_DBG(context).handler->remote_breakpoint(&(XG_DBG(context)), XG_BASE(stack), fse->filename, fse->lineno, XDEBUG_BREAK, NULL, 0, NULL)) {
 							return 0;
 						}
 					} else {
 						XG_DBG(context).do_break = 1;
-						XG_DBG(context).pending_breakpoint = extra_brk_info;
 					}
 				}
 			}
@@ -482,7 +413,7 @@ static int handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 
 void xdebug_debugger_handle_breakpoints(function_stack_entry *fse, int breakpoint_type)
 {
-	if (xdebug_is_debug_connection_active() && XG_DBG(breakpoints_allowed)) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(breakpoints_allowed)) {
 		if (!handle_breakpoints(fse, breakpoint_type)) {
 			xdebug_mark_debug_connection_not_active();
 		}
@@ -491,7 +422,7 @@ void xdebug_debugger_handle_breakpoints(function_stack_entry *fse, int breakpoin
 
 static size_t xdebug_ub_write(const char *string, size_t length)
 {
-	if (xdebug_is_debug_connection_active()) {
+	if (xdebug_is_debug_connection_active_for_current_pid()) {
 		if (-1 == XG_DBG(context).handler->remote_stream_output(string, length)) {
 			return 0;
 		}
@@ -544,7 +475,24 @@ void xdebug_debugger_rinit(void)
 {
 	char *idekey;
 
-	xdebug_disable_opcache_optimizer();
+/* PHP Bug #77287 causes Xdebug to segfault if OPcache has the "compact
+ * literals" optimisation turned on. So force the optimisation off for PHP
+ * 7.3.0 and 7.3.1.
+ *
+ * Otherwise, only turn off optimisation when we're debugging. */
+#if PHP_VERSION_ID >= 70300 && PHP_VERSION_ID <= 70301
+	{
+#else
+	if (XINI_DBG(remote_enable)) {
+#endif
+		zend_string *key = zend_string_init(ZEND_STRL("opcache.optimization_level"), 1);
+		zend_string *value = zend_string_init(ZEND_STRL("0"), 1);
+
+		zend_alter_ini_entry(key, value, ZEND_INI_SYSTEM, ZEND_INI_STAGE_STARTUP);
+
+		zend_string_release(key);
+		zend_string_release(value);
+	}
 
 	/* Get the ide key for this session */
 	XG_DBG(ide_key) = NULL;
@@ -558,6 +506,7 @@ void xdebug_debugger_rinit(void)
 
 	XG_DBG(no_exec) = 0;
 	xdebug_lib_set_active_symbol_table(NULL);
+	xdebug_lib_set_active_object(NULL);
 
 	/* Check if we have this special get variable that stops a debugging
 	 * request without executing any code */
@@ -573,7 +522,7 @@ void xdebug_debugger_rinit(void)
 			)
 			&& !SG(headers_sent)
 		) {
-			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), (char*) "", 0, 0, "/", 1, NULL, 0, 0, 1, 0);
+			xdebug_setcookie("XDEBUG_SESSION", sizeof("XDEBUG_SESSION"), (char*) "", 0, time(NULL) + XINI_DBG(remote_cookie_expire_time), "/", 1, NULL, 0, 0, 1, 0);
 			XG_DBG(no_exec) = 1;
 		}
 		zend_string_release(stop_no_exec);
@@ -586,22 +535,16 @@ void xdebug_debugger_rinit(void)
 	XG_DBG(breakable_lines_map) = xdebug_hash_alloc(2048, (xdebug_hash_dtor_t) xdebug_line_list_dtor);
 	XG_DBG(function_count) = 0;
 	XG_DBG(class_count) = 0;
+	XG_DBG(remote_log_file) = NULL;
 
 	/* Initialize some debugger context properties */
 	XG_DBG(context).program_name   = NULL;
-	XG_DBG(context).list.last_filename = NULL;
+	XG_DBG(context).list.last_file = NULL;
 	XG_DBG(context).list.last_line = 0;
 	XG_DBG(context).do_break       = 0;
-	XG_DBG(context).pending_breakpoint = NULL;
 	XG_DBG(context).do_step        = 0;
 	XG_DBG(context).do_next        = 0;
 	XG_DBG(context).do_finish      = 0;
-	XG_DBG(context).do_connect_to_client = 0;
-
-	/* Statistics and diagnostics */
-	XG_DBG(context).connected_hostname = NULL;
-	XG_DBG(context).connected_port = 0;
-	XG_DBG(context).detached_message = NULL;
 }
 
 void xdebug_debugger_post_deactivate(void)
@@ -611,7 +554,7 @@ void xdebug_debugger_post_deactivate(void)
 		xdebug_close_socket(XG_DBG(context).socket);
 	}
 	if (XG_DBG(context).program_name) {
-		zend_string_release(XG_DBG(context).program_name);
+		xdfree(XG_DBG(context).program_name);
 	}
 
 	if (XG_DBG(ide_key)) {
@@ -619,23 +562,13 @@ void xdebug_debugger_post_deactivate(void)
 		XG_DBG(ide_key) = NULL;
 	}
 
-	if (XG_DBG(context.list.last_filename)) {
-		zend_string_release(XG_DBG(context).list.last_filename);
-		XG_DBG(context).list.last_filename = NULL;
+	if (XG_DBG(context.list.last_file)) {
+		xdfree(XG_DBG(context).list.last_file);
+		XG_DBG(context).list.last_file = NULL;
 	}
 
 	xdebug_hash_destroy(XG_DBG(breakable_lines_map));
 	XG_DBG(breakable_lines_map) = NULL;
-
-	if (XG_DBG(context).connected_hostname) {
-		xdfree(XG_DBG(context).connected_hostname);
-		XG_DBG(context).connected_hostname = NULL;
-	}
-
-	if (XG_DBG(context).detached_message) {
-		xdfree(XG_DBG(context).detached_message);
-		XG_DBG(context).detached_message = NULL;
-	}
 }
 
 xdebug_set *xdebug_debugger_get_breakable_lines_from_oparray(zend_op_array *opa)
@@ -706,16 +639,6 @@ static void add_function_to_lines_list(xdebug_lines_list *lines_list, zend_op_ar
 	}
 	lines_list->functions[lines_list->count] = map_item;
 	lines_list->count++;
-
-#if PHP_VERSION_ID >= 80100
-	if (opa->num_dynamic_func_defs) {
-		uint32_t i;
-
-		for (i = 0; i < opa->num_dynamic_func_defs; i++) {
-			add_function_to_lines_list(lines_list, opa->dynamic_func_defs[i]);
-		}
-	}
-#endif
 }
 /* }}} */
 
@@ -750,11 +673,7 @@ void xdebug_debugger_compile_file(zend_op_array *op_array)
 	zend_class_entry *class_entry;
 	xdebug_lines_list *file_function_lines_list;
 
-	RETURN_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
-
-	/* The breakable_lines_map can not be set if another extension compiles
-	 * scripts during RINIT */
-	if (!XG_DBG(breakable_lines_map)) {
+	if (!XINI_DBG(remote_enable) || !XG_DBG(breakable_lines_map)) {
 		return;
 	}
 
@@ -784,7 +703,7 @@ void xdebug_debugger_compile_file(zend_op_array *op_array)
 
 	add_function_to_lines_list(file_function_lines_list, op_array);
 
-	if (!xdebug_is_debug_connection_active()) {
+	if (!xdebug_is_debug_connection_active_for_current_pid()) {
 		return;
 	}
 
@@ -805,7 +724,7 @@ static void resolve_breakpoints_for_eval(int eval_id, zend_op_array *opa)
 
 	resolve_breakpoints_for_function(lines_list, opa);
 
-	if (!xdebug_is_debug_connection_active()) {
+	if (!xdebug_is_debug_connection_active_for_current_pid()) {
 		zend_string_release(eval_string);
 		xdfree(eval_filename);
 		return;
@@ -822,84 +741,19 @@ static void resolve_breakpoints_for_eval(int eval_id, zend_op_array *opa)
 
 void xdebug_debugger_register_eval(function_stack_entry *fse)
 {
-	if (xdebug_is_debug_connection_active() && XG_DBG(context).handler->register_eval_id) {
+	if (xdebug_is_debug_connection_active_for_current_pid() && XG_DBG(context).handler->register_eval_id) {
 		int eval_id = XG_DBG(context).handler->register_eval_id(&(XG_DBG(context)), fse);
 
 		resolve_breakpoints_for_eval(eval_id, fse->op_array);
 	}
 }
 
-void xdebug_debugger_restart_if_pid_changed()
-{
-	zend_ulong pid;
-
-	if (!xdebug_is_debug_connection_active()) {
-		return;
-	}
-
-	pid = xdebug_get_pid();
-
-	/* Start debugger if previously a connection was established and this
-	 * process no longer has the same PID */
-	if (XG_DBG(remote_connection_pid) != pid) {
-		xdebug_restart_debugger();
-	}
-}
 
 PHP_FUNCTION(xdebug_break)
 {
-	RETURN_FALSE_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
-
-	xdebug_debug_init_if_requested_on_xdebug_break();
-
-	if (!xdebug_is_debug_connection_active()) {
-		RETURN_FALSE;
-	}
-
-	register_compiled_variables();
+	/* Start JIT if requested and not yet enabled */
+	xdebug_do_jit();
 
 	XG_DBG(context).do_break = 1;
-	XG_DBG(context).pending_breakpoint = NULL;
-
-	RETURN_TRUE;
-}
-
-PHP_FUNCTION(xdebug_connect_to_client)
-{
-	RETURN_FALSE_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
-
-	XG_DBG(context).do_connect_to_client = 1;
-
-	RETURN_TRUE;
-}
-
-PHP_FUNCTION(xdebug_is_debugger_active)
-{
-	RETURN_BOOL(xdebug_is_debug_connection_active());
-}
-
-PHP_FUNCTION(xdebug_notify)
-{
-	function_stack_entry *fse;
-	zval *data;
-
-	RETURN_FALSE_IF_MODE_IS_NOT(XDEBUG_MODE_STEP_DEBUG);
-
-	if (!xdebug_is_debug_connection_active()) {
-		RETURN_FALSE;
-	}
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &data) == FAILURE) {
-		return;
-	}
-
-	fse = xdebug_get_stack_frame(0);
-
-	XG_DBG(context).handler->user_notification(
-		&(XG_DBG(context)),
-		fse->filename, fse->lineno,
-		data
-	);
-
 	RETURN_TRUE;
 }
