@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 #include "php_xdebug.h"
 
@@ -125,7 +123,7 @@ void xdebug_log_stack(const char *error_type_str, char *buffer, const char *erro
 			sent_variables--;
 		}
 
-		tmp_name = xdebug_show_fname(fse->function, 0, 0);
+		tmp_name = xdebug_show_fname(fse->function, XDEBUG_SHOW_FNAME_DEFAULT);
 		xdebug_str_add_fmt(&log_buffer, "PHP %3d. %s(", fse->level, tmp_name);
 		xdfree(tmp_name);
 
@@ -396,7 +394,7 @@ void xdebug_append_printable_stack(xdebug_str *str, int html)
 		if (xdebug_is_stack_frame_filtered(XDEBUG_FILTER_STACK, fse)) {
 			continue;
 		}
-		tmp_name = xdebug_show_fname(fse->function, html, 0);
+		tmp_name = xdebug_show_fname(fse->function, html ? XDEBUG_SHOW_FNAME_ALLOW_HTML : XDEBUG_SHOW_FNAME_DEFAULT);
 		if (html) {
 			xdebug_str_add_fmt(str, formats[3], fse->level, XDEBUG_SECONDS_SINCE_START(fse->nanotime), fse->memory, tmp_name);
 		} else {
@@ -627,7 +625,11 @@ static void clear_last_error()
 		PG(last_error_message) = NULL;
 	}
 	if (PG(last_error_file)) {
+# if PHP_VERSION_ID >= 80100
+		zend_string_release(PG(last_error_file));
+# else
 		free(PG(last_error_file));
+# endif
 		PG(last_error_file) = NULL;
 	}
 }
@@ -648,7 +650,10 @@ static void clear_last_error()
 #endif
 
 /* Error callback for formatting stack traces */
-#if PHP_VERSION_ID >= 80000
+#if PHP_VERSION_ID >= 80100
+void xdebug_develop_error_cb(int orig_type, zend_string *error_filename, const unsigned int error_lineno, zend_string *message)
+{
+#elif PHP_VERSION_ID >= 80000
 void xdebug_develop_error_cb(int orig_type, const char *error_filename, const unsigned int error_lineno, zend_string *message)
 {
 #else
@@ -676,13 +681,20 @@ void xdebug_develop_error_cb(int orig_type, const char *error_filename, const un
 			 * be NULL if PG(last_error_message) is not NULL */
 
 #if PHP_VERSION_ID >= 80000
-			if (zend_string_equals(PG(last_error_message), message)
+			if (!zend_string_equals(PG(last_error_message), message)
 #else
-			if (strcmp(PG(last_error_message), buffer)
+			if (strcmp(PG(last_error_message), buffer) != 0
 #endif
-					|| (!PG(ignore_repeated_source)
-							&& ((PG(last_error_lineno) != (int)error_lineno)
-									|| strcmp(PG(last_error_file), error_filename)))) {
+				||
+					(!PG(ignore_repeated_source) && (
+						(PG(last_error_lineno) != (int)error_lineno) ||
+#if PHP_VERSION_ID >= 80100
+						!zend_string_equals(PG(last_error_file), error_filename)
+#else
+						strcmp(PG(last_error_file), error_filename) != 0
+#endif
+					))
+			) {
 					display = 1;
 			} else {
 					display = 0;
@@ -758,20 +770,28 @@ void xdebug_develop_error_cb(int orig_type, const char *error_filename, const un
 	if (display) {
 		clear_last_error();
 		if (!error_filename) {
+# if PHP_VERSION_ID >= 80100
+			error_filename = zend_string_init(ZEND_STRL("Unknown"), 0);
+# else
 			error_filename = "Unknown";
+# endif
 		}
 		PG(last_error_type) = type;
 # if PHP_VERSION_ID >= 80000
 		PG(last_error_message) = zend_string_copy(message);
-#else
+# else
 		PG(last_error_message) = strdup(buffer);
-#endif
+# endif
+# if PHP_VERSION_ID >= 80100
+		PG(last_error_file) = zend_string_copy(error_filename);
+# else
 		PG(last_error_file) = strdup(error_filename);
+# endif
 		PG(last_error_lineno) = error_lineno;
 	}
 #endif
 
-	if ((EG(error_reporting | XINI_DEV(force_error_reporting))) & type) {
+	if ((EG(error_reporting) | XINI_DEV(force_error_reporting)) & type) {
 		/* Log to logger */
 		if (PG(log_errors)) {
 
@@ -784,7 +804,9 @@ void xdebug_develop_error_cb(int orig_type, const char *error_filename, const un
 #endif
 			}
 #endif
-#if PHP_VERSION_ID >= 80000
+#if PHP_VERSION_ID >= 80100
+			xdebug_log_stack(error_type_str, ZSTR_VAL(message), ZSTR_VAL(error_filename), error_lineno);
+#elif PHP_VERSION_ID >= 80000
 			xdebug_log_stack(error_type_str, ZSTR_VAL(message), error_filename, error_lineno);
 #else
 			xdebug_log_stack(error_type_str, buffer, error_filename, error_lineno);
@@ -816,7 +838,9 @@ void xdebug_develop_error_cb(int orig_type, const char *error_filename, const un
 		if ((PG(display_errors) || XINI_DEV(force_display_errors)) && !PG(during_request_startup)) {
 			char *printable_stack;
 
-#if PHP_VERSION_ID >= 80000
+#if PHP_VERSION_ID >= 80100
+			printable_stack = xdebug_handle_stack_trace(type, error_type_str, ZSTR_VAL(error_filename), error_lineno, ZSTR_VAL(message));
+#elif PHP_VERSION_ID >= 80000
 			printable_stack = xdebug_handle_stack_trace(type, error_type_str, error_filename, error_lineno, ZSTR_VAL(message));
 #else
 			printable_stack = xdebug_handle_stack_trace(type, error_type_str, error_filename, error_lineno, buffer);
@@ -830,7 +854,9 @@ void xdebug_develop_error_cb(int orig_type, const char *error_filename, const un
 			}
 		} else if (XG_LIB(do_collect_errors)) {
 			char *printable_stack;
-#if PHP_VERSION_ID >= 80000
+#if PHP_VERSION_ID >= 80100
+			printable_stack = xdebug_get_printable_stack(PG(html_errors), type, ZSTR_VAL(message), ZSTR_VAL(error_filename), error_lineno, 1);
+#elif PHP_VERSION_ID >= 80000
 			printable_stack = xdebug_get_printable_stack(PG(html_errors), type, ZSTR_VAL(message), error_filename, error_lineno, 1);
 #else
 			printable_stack = xdebug_get_printable_stack(PG(html_errors), type, buffer, error_filename, error_lineno, 1);
@@ -840,7 +866,11 @@ void xdebug_develop_error_cb(int orig_type, const char *error_filename, const un
 	}
 
 	{
+#if PHP_VERSION_ID >= 80100
+		zend_string *tmp_error_filename = zend_string_copy(error_filename);
+#else
 		zend_string *tmp_error_filename = zend_string_init(error_filename, strlen(error_filename), 0);
+#endif
 #if PHP_VERSION_ID >= 80000
 		xdebug_debugger_error_cb(tmp_error_filename, error_lineno, type, error_type_str, ZSTR_VAL(message));
 #else
@@ -999,6 +1029,7 @@ PHP_FUNCTION(xdebug_get_function_stack)
 	unsigned int          j;
 	zval                 *frame;
 	zval                 *params;
+	int                   variadic_opened = 0;
 
 	if (!XDEBUG_MODE_IS(XDEBUG_MODE_DEVELOP)) {
 		php_error(E_WARNING, "Function must be enabled in php.ini by setting 'xdebug.mode' to 'develop'");
@@ -1031,9 +1062,9 @@ PHP_FUNCTION(xdebug_get_function_stack)
 		if (fse->function.function) {
 			add_assoc_string_ex(frame, "function", HASH_KEY_SIZEOF("function"), fse->function.function);
 		}
-		if (fse->function.class_name) {
+		if (fse->function.object_class) {
 			add_assoc_string_ex(frame, "type",     HASH_KEY_SIZEOF("type"),     (char*) (fse->function.type == XFUNC_STATIC_MEMBER ? "static" : "dynamic"));
-			add_assoc_str_ex(frame,    "class",    HASH_KEY_SIZEOF("class"),    zend_string_copy(fse->function.class_name));
+			add_assoc_str_ex(frame,    "class",    HASH_KEY_SIZEOF("class"),    zend_string_copy(fse->function.object_class));
 		}
 		add_assoc_str_ex(frame, "file", HASH_KEY_SIZEOF("file"), zend_string_copy(fse->filename));
 		add_assoc_long_ex(frame, "line", HASH_KEY_SIZEOF("line"), fse->lineno);
@@ -1044,7 +1075,6 @@ PHP_FUNCTION(xdebug_get_function_stack)
 		add_assoc_zval_ex(frame, "params", HASH_KEY_SIZEOF("params"), params);
 
 		for (j = 0; j < sent_variables; j++) {
-			int         variadic_opened = 0;
 			xdebug_str *argument = NULL;
 
 			if (fse->var[j].is_variadic) {
@@ -1071,7 +1101,7 @@ PHP_FUNCTION(xdebug_get_function_stack)
 			if (fse->var[j].name && !variadic_opened && argument) {
 				add_assoc_stringl_ex(params, ZSTR_VAL(fse->var[j].name), ZSTR_LEN(fse->var[j].name), argument->d, argument->l);
 			} else {
-				add_index_stringl(params, j - 1, argument->d, argument->l);
+				add_index_stringl(params, j - variadic_opened, argument->d, argument->l);
 			}
 			if (argument) {
 				xdebug_str_free(argument);
