@@ -12,6 +12,9 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
+   | Authors: Benjamin Eberlei <kontakt@beberlei.de>					  |
+   |          Derick Rethans <derick@xdebug.org>                          |
+   +----------------------------------------------------------------------+
  */
 
 #ifndef __XDEBUG_GC_STATS_PRIVATE_H__
@@ -25,7 +28,7 @@ typedef struct _xdebug_gc_run {
 	zend_long    memory_before;
 	zend_long    memory_after;
 	char        *function_name;
-	zend_string *class_name;
+	char        *class_name;
 } xdebug_gc_run;
 
 #define XINI_GCSTATS(v)  (XG(settings.gc_stats.v))
