@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2022 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include "php_xdebug.h"
@@ -59,6 +57,9 @@ void xdebug_library_minit(void)
 	xdebug_set_opcode_multi_handler(ZEND_ASSIGN);
 	xdebug_set_opcode_multi_handler(ZEND_ASSIGN_DIM);
 	xdebug_set_opcode_multi_handler(ZEND_ASSIGN_OBJ);
+#if PHP_VERSION_ID >= 70400
+	xdebug_set_opcode_multi_handler(ZEND_ASSIGN_STATIC_PROP);
+#endif
 	xdebug_set_opcode_multi_handler(ZEND_QM_ASSIGN);
 	xdebug_set_opcode_multi_handler(ZEND_INCLUDE_OR_EVAL);
 }
@@ -231,14 +232,14 @@ int xdebug_lib_set_start_with_request(char *value)
 	return 0;
 }
 
-int xdebug_lib_start_with_request(void)
+int xdebug_lib_start_with_request(int for_mode)
 {
 	if (XG_LIB(start_with_request) == XDEBUG_START_WITH_REQUEST_YES) {
 		return 1;
 	}
 
 	if (XG_LIB(start_with_request) == XDEBUG_START_WITH_REQUEST_DEFAULT) {
-		if (XDEBUG_MODE_IS(XDEBUG_MODE_PROFILING)) {
+		if (for_mode == XDEBUG_MODE_PROFILING && XDEBUG_MODE_IS(XDEBUG_MODE_PROFILING)) {
 			return 1;
 		}
 	}
@@ -288,6 +289,25 @@ int xdebug_lib_start_upon_error(void)
 	return 0;
 }
 
+const char *xdebug_lib_mode_from_value(int mode)
+{
+	switch (mode) {
+		case XDEBUG_MODE_DEVELOP:
+			return "develop";
+		case XDEBUG_MODE_COVERAGE:
+			return "coverage";
+		case XDEBUG_MODE_STEP_DEBUG:
+			return "debug";
+		case XDEBUG_MODE_GCSTATS:
+			return "gcstats";
+		case XDEBUG_MODE_PROFILING:
+			return "profile";
+		case XDEBUG_MODE_TRACING:
+			return "trace";
+		default:
+			return "?";
+	}
+}
 
 static const char *find_in_globals(const char *element)
 {
@@ -315,41 +335,120 @@ static const char *find_in_globals(const char *element)
 	return NULL;
 }
 
-static int trigger_enabled(int for_mode, char **found_trigger_value)
+int xdebug_lib_has_shared_secret(void)
 {
 	char *shared_secret = XINI_LIB(trigger_value);
+
+	if (shared_secret != NULL && shared_secret[0] != '\0') {
+		return 1;
+	}
+
+	return 0;
+}
+
+static int does_shared_secret_match_single(int mode, const char *trimmed_trigger_value, const char *trimmed_shared_secret, char **found_trigger_value)
+{
+	if (strcmp(trimmed_shared_secret, trimmed_trigger_value) == 0) {
+		xdebug_log_ex(XLOG_CHAN_CONFIG, XLOG_DEBUG, "TRGSEC-MATCH", "The trigger value '%s' matched the shared secret '%s' for mode '%s'", trimmed_trigger_value, trimmed_shared_secret, xdebug_lib_mode_from_value(mode));
+
+		if (found_trigger_value != NULL) {
+			*found_trigger_value = xdstrdup(trimmed_trigger_value);
+		}
+
+		return 1;
+	}
+
+	return 0;
+}
+
+static int does_shared_secret_match(int mode, const char *trigger_name, const char *trigger_value, char **found_trigger_value)
+{
+	int         retval = 0;
+	const char *shared_secret = XINI_LIB(trigger_value);
+	char       *trimmed_trigger_value = xdebug_trim(trigger_value);
+
+	/* Check we have a multi-value-shared secret setting */
+	if (strchr(shared_secret, ',') != NULL) {
+		int         i;
+		xdebug_arg *values = xdebug_arg_ctor();
+
+		xdebug_log_ex(XLOG_CHAN_CONFIG, XLOG_DEBUG, "TRGSEC-MULT", "The shared secret (xdebug.trigger_value) is multi-value for mode '%s'", xdebug_lib_mode_from_value(mode));
+
+		xdebug_explode(",", shared_secret, values, -1);
+
+		for (i = 0; i < values->c; i++) {
+			char *trimmed_shared_secret = xdebug_trim(values->args[i]);
+
+			retval = does_shared_secret_match_single(mode, trimmed_trigger_value, trimmed_shared_secret, found_trigger_value);
+
+			xdfree(trimmed_shared_secret);
+
+			/* Jump out of the loop if we found a match */
+			if (retval != 0) {
+				break;
+			}
+		}
+
+		xdebug_arg_dtor(values);
+
+		if (retval == 0) {
+			xdebug_log_ex(XLOG_CHAN_CONFIG, XLOG_WARN, "TRGSEC-MNO", "The trigger value '%s', as set through '%s', did not match any of the shared secrets (xdebug.trigger_value) for mode '%s'", trimmed_trigger_value, trigger_name, xdebug_lib_mode_from_value(mode));
+		}
+	} else {
+		char *trimmed_shared_secret = xdebug_trim(shared_secret);
+
+		retval = does_shared_secret_match_single(mode, trimmed_trigger_value, trimmed_shared_secret, found_trigger_value);
+
+		xdfree(trimmed_shared_secret);
+
+		if (retval == 0) {
+			xdebug_log_ex(XLOG_CHAN_CONFIG, XLOG_WARN, "TRGSEC-NO", "The trigger value '%s', as set through '%s', did not match the shared secret (xdebug.trigger_value) for mode '%s'", trimmed_trigger_value, trigger_name, xdebug_lib_mode_from_value(mode));
+		}
+	}
+
+	xdfree(trimmed_trigger_value);
+
+	return retval;
+}
+
+static int trigger_enabled(int for_mode, char **found_trigger_value)
+{
 	const char *trigger_value = NULL;
+	const char *trigger_name = "XDEBUG_TRIGGER";
+
+	xdebug_log(XLOG_CHAN_CONFIG, XLOG_DEBUG, "Checking if trigger 'XDEBUG_TRIGGER' is enabled for mode '%s'", xdebug_lib_mode_from_value(for_mode));
 
 	/* First we check for the generic 'XDEBUG_TRIGGER' option */
-	trigger_value = find_in_globals("XDEBUG_TRIGGER");
+	trigger_value = find_in_globals(trigger_name);
 
 	/* If not found, we fall back to the per-mode name for backwards compatibility reasons */
 	if (!trigger_value) {
-		const char *fallback_name = NULL;
-
 		if (XDEBUG_MODE_IS(XDEBUG_MODE_PROFILING) && (for_mode == XDEBUG_MODE_PROFILING)) {
-			fallback_name = "XDEBUG_PROFILE";
+			trigger_name = "XDEBUG_PROFILE";
 		} else if (XDEBUG_MODE_IS(XDEBUG_MODE_TRACING) && (for_mode == XDEBUG_MODE_TRACING)) {
-			fallback_name = "XDEBUG_TRACE";
+			trigger_name = "XDEBUG_TRACE";
 		} else if (XDEBUG_MODE_IS(XDEBUG_MODE_STEP_DEBUG) && (for_mode == XDEBUG_MODE_STEP_DEBUG)) {
-			fallback_name = "XDEBUG_SESSION";
+			trigger_name = "XDEBUG_SESSION";
 		}
 
-		if (fallback_name) {
-			trigger_value = find_in_globals(fallback_name);
+		if (trigger_name) {
+			xdebug_log(XLOG_CHAN_CONFIG, XLOG_INFO, "Trigger value for 'XDEBUG_TRIGGER' not found, falling back to '%s'", trigger_name);
+			trigger_value = find_in_globals(trigger_name);
 		}
 	}
 
 	if (!trigger_value) {
+		xdebug_log(XLOG_CHAN_CONFIG, XLOG_INFO, "Trigger value for '%s' not found, so not activating", trigger_name);
+
 		if (found_trigger_value != NULL) {
 			*found_trigger_value = NULL;
 		}
 		return 0;
 	}
 
-	/* If the configured shared secret trigger value is empty, then it always
-	 * triggers */
-	if (shared_secret == NULL || shared_secret[0] == '\0') {
+	/* If there is no configured shared secret trigger, always trigger */
+	if (!xdebug_lib_has_shared_secret()) {
+		xdebug_log(XLOG_CHAN_CONFIG, XLOG_INFO, "No shared secret: Activating");
 		if (found_trigger_value != NULL) {
 			*found_trigger_value = xdstrdup(trigger_value);
 		}
@@ -358,10 +457,7 @@ static int trigger_enabled(int for_mode, char **found_trigger_value)
 
 	/* Check if the configured trigger value matches the one found in the
 	 * trigger element */
-	if (strcmp(shared_secret, trigger_value) == 0) {
-		if (found_trigger_value != NULL) {
-			*found_trigger_value = xdstrdup(trigger_value);
-		}
+	if (does_shared_secret_match(for_mode, trigger_name, trigger_value, found_trigger_value)) {
 		return 1;
 	}
 
@@ -610,6 +706,18 @@ void xdebug_llist_string_dtor(void *dummy, void *elem)
 	}
 }
 
+char* xdebug_wrap_location_around_function_name(const char *prefix, zend_op_array *opa, char *fname)
+{
+	return xdebug_sprintf(
+		"%s{%s:%s:%d-%d}",
+		fname,
+		prefix,
+		opa->filename->val,
+		opa->line_start,
+		opa->line_end
+	);
+}
+
 char* xdebug_wrap_closure_location_around_function_name(zend_op_array *opa, char *fname)
 {
 	xdebug_str tmp = XDEBUG_STR_INITIALIZER;
@@ -649,6 +757,10 @@ void xdebug_lib_register_compiled_variables(function_stack_entry *fse, zend_op_a
 		fse->declared_vars = xdebug_llist_alloc(xdebug_declared_var_dtor);
 	}
 
+	if (!op_array->vars) {
+		return;
+	}
+
 	/* gather used variables from compiled vars information */
 	while (i < (unsigned int) op_array->last_var) {
 		xdebug_llist_insert_next(fse->declared_vars, XDEBUG_LLIST_TAIL(fse->declared_vars), xdebug_str_create(STR_NAME_VAL(op_array->vars[i]), STR_NAME_LEN(op_array->vars[i])));
