@@ -21,8 +21,6 @@
 
 #include <stddef.h>
 
-#include "php.h"
-
 #include "mm.h"
 
 #define XDEBUG_STR_INITIALIZER { 0, 0, NULL }
@@ -40,13 +38,7 @@ typedef struct xdebug_str {
 void xdebug_str_add(xdebug_str *xs, const char *str, int f);
 void xdebug_str_addl(xdebug_str *xs, const char *str, int le, int f);
 void xdebug_str_add_str(xdebug_str *xs, const xdebug_str *str);
-void xdebug_str_add_zstr(xdebug_str *xs, const zend_string *str);
 void xdebug_str_addc(xdebug_str *xs, char letter);
-void xdebug_str_add_uint64(xdebug_str *xs, uint64_t num);
-void xdebug_str_add_fmt(xdebug_str *xs, const char *fmt, ...);
-#define xdebug_str_add_literal(s,l) xdebug_str_addl((s), (l), sizeof(l)-1, 0)
-#define xdebug_str_add_const(s,l) xdebug_str_addl((s), (l), strlen(l), 0)
-
 void xdebug_str_chop(xdebug_str *xs, size_t c);
 
 xdebug_str *xdebug_str_new(void);
@@ -56,7 +48,7 @@ xdebug_str *xdebug_str_copy(xdebug_str *orig);
 void xdebug_str_destroy(xdebug_str *s);
 void xdebug_str_free(xdebug_str *s);
 
-char* xdebug_sprintf(const char* fmt, ...);
+char* xdebug_sprintf (const char* fmt, ...);
 char* xdebug_strndup(const char *s, int length);
 
 #endif
