@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2022 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,31 +12,34 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include <stdlib.h>
 #include <string.h>
+
 #ifndef WIN32
-#include <unistd.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/time.h>
-#include <sys/resource.h>
-#include <sys/file.h>
+# include <unistd.h>
+# include <sys/types.h>
+# include <sys/socket.h>
+# include <sys/resource.h>
+# include <sys/file.h>
 #else
-#define PATH_MAX MAX_PATH
-#include <winsock2.h>
-#include <io.h>
-#include "win32/time.h"
-#include <process.h>
+# define PATH_MAX MAX_PATH
+# include <winsock2.h>
+# include <io.h>
+# include <process.h>
 #endif
+
 #include "php_xdebug.h"
+
 #include "mm.h"
 #include "crc32.h"
 #include "str.h"
+
+#include "lib_private.h"
 #include "usefulstuff.h"
+#include "log.h"
+
 #include "ext/standard/php_lcg.h"
 #include "ext/standard/flock_compat.h"
 #include "main/php_ini.h"
@@ -52,6 +55,29 @@
 
 ZEND_EXTERN_MODULE_GLOBALS(xdebug)
 
+xdebug_arg *xdebug_arg_ctor(void)
+{
+	xdebug_arg *tmp = /*(xdebug_arg*)*/ xdmalloc(sizeof(xdebug_arg));
+
+	tmp->args = NULL;
+	tmp->c    = 0;
+
+	return tmp;
+}
+
+void xdebug_arg_dtor(xdebug_arg *arg)
+{
+	int i;
+
+	for (i = 0; i < arg->c; i++) {
+		xdfree(arg->args[i]);
+	}
+	if (arg->args) {
+		xdfree(arg->args);
+	}
+	xdfree(arg);
+}
+
 xdebug_str* xdebug_join(const char *delim, xdebug_arg *args, int begin, int end)
 {
 	int         i;
@@ -71,9 +97,9 @@ xdebug_str* xdebug_join(const char *delim, xdebug_arg *args, int begin, int end)
 	return ret;
 }
 
-void xdebug_explode(const char *delim, char *str, xdebug_arg *args, int limit)
+void xdebug_explode(const char *delim, const char *str, xdebug_arg *args, int limit)
 {
-	char *p1, *p2, *endp;
+	const char *p1, *p2, *endp;
 
 	endp = str + strlen(str);
 
@@ -106,9 +132,9 @@ void xdebug_explode(const char *delim, char *str, xdebug_arg *args, int limit)
 	}
 }
 
-char* xdebug_memnstr(char *haystack, const char *needle, int needle_len, char *end)
+const char* xdebug_memnstr(const char *haystack, const char *needle, int needle_len, const char *end)
 {
-	char *p = haystack;
+	const char *p = haystack;
 	char first = *needle;
 
 	/* let end point to the last character where needle may start */
@@ -139,35 +165,32 @@ char* xdebug_strrstr(const char* haystack, const char* needle)
 	return loc;
 }
 
-double xdebug_get_utime(void)
+char *xdebug_trim(const char *str)
 {
-#ifdef HAVE_GETTIMEOFDAY
-	struct timeval tp;
-	long sec = 0L;
-	double msec = 0.0;
+	char *trimmed = NULL, *begin = (char *) str, *end = NULL;
 
-	if (gettimeofday((struct timeval *) &tp, NULL) == 0) {
-		sec = tp.tv_sec;
-		msec = (double) (tp.tv_usec / MICRO_IN_SEC);
-
-		if (msec >= 1.0) {
-			msec -= (long) msec;
-		}
-		return msec + sec;
+	/* trim leading space */
+	while (isspace((unsigned char) *begin)) {
+		++begin;
 	}
-#endif
-	return 0;
-}
 
-char* xdebug_get_time(void)
-{
-	time_t cur_time;
-	char  *str_time;
+	/* All spaces */
+	if (*begin == '\0') {
+		return xdstrdup("");
+	}
 
-	str_time = xdmalloc(24);
-	cur_time = time(NULL);
-	strftime(str_time, 24, "%Y-%m-%d %H:%M:%S", gmtime (&cur_time));
-	return str_time;
+	/* trim trailing space */
+	end = begin + strlen(begin) - 1;
+	while (end > begin && isspace((unsigned char) *end)) {
+		--end;
+	}
+	end++;
+
+	trimmed = xdmalloc(end - begin + 1);
+	memcpy(trimmed, begin, (end - begin));
+	trimmed[(end - begin)] = '\0';
+
+	return trimmed;
 }
 
 /* not all versions of php export this */
@@ -240,11 +263,11 @@ char *xdebug_raw_url_encode(char const *s, int len, int *new_length, int skip_sl
 }
 
 /* fake URI's per IETF RFC 1738 and 2396 format */
-char *xdebug_path_from_url(const char *fileurl)
+char *xdebug_path_from_url(zend_string *fileurl)
 {
 	/* deal with file: url's */
 	char *dfp = NULL;
-	const char *fp = NULL, *efp = fileurl;
+	const char *fp = NULL, *efp = ZSTR_VAL(fileurl);
 #ifdef PHP_WIN32
 	int l = 0;
 	int i;
@@ -272,7 +295,7 @@ char *xdebug_path_from_url(const char *fileurl)
 		}
 #endif
 	} else {
-		ret = xdstrdup(fileurl);
+		ret = xdstrdup(ZSTR_VAL(fileurl));
 	}
 
 	free(dfp);
@@ -280,19 +303,19 @@ char *xdebug_path_from_url(const char *fileurl)
 }
 
 /* fake URI's per IETF RFC 1738 and 2396 format */
-char *xdebug_path_to_url(const char *fileurl)
+char *xdebug_path_to_url(zend_string *fileurl)
 {
 	int l, i, new_len;
 	char *tmp = NULL;
 	char *encoded_fileurl;
 
 	/* encode the url */
-	encoded_fileurl = xdebug_raw_url_encode(fileurl, strlen(fileurl), &new_len, 1);
+	encoded_fileurl = xdebug_raw_url_encode(ZSTR_VAL(fileurl), ZSTR_LEN(fileurl), &new_len, 1);
 
-	if (strncmp(fileurl, "phar://", 7) == 0) {
+	if (strncmp(ZSTR_VAL(fileurl), "phar://", 7) == 0) {
 		/* ignore, phar is cool */
-		tmp = xdstrdup(fileurl);
-	} else if (fileurl[0] != '/' && fileurl[0] != '\\' && fileurl[1] != ':') {
+		tmp = xdstrdup(ZSTR_VAL(fileurl));
+	} else if (ZSTR_VAL(fileurl)[0] != '/' && ZSTR_VAL(fileurl)[0] != '\\' && ZSTR_VAL(fileurl)[1] != ':') {
 		/* convert relative paths */
 		cwd_state new_state;
 		char cwd[MAXPATHLEN];
@@ -306,21 +329,21 @@ char *xdebug_path_to_url(const char *fileurl)
 		new_state.cwd = estrdup(cwd);
 		new_state.cwd_length = strlen(cwd);
 
-		if (!virtual_file_ex(&new_state, fileurl, NULL, 1)) {
+		if (!virtual_file_ex(&new_state, ZSTR_VAL(fileurl), NULL, 1)) {
 			char *s = estrndup(new_state.cwd, new_state.cwd_length);
 			tmp = xdebug_sprintf("file://%s",s);
 			efree(s);
 		}
 		efree(new_state.cwd);
 
-	} else if (fileurl[1] == '/' || fileurl[1] == '\\') {
+	} else if (ZSTR_VAL(fileurl)[1] == '/' || ZSTR_VAL(fileurl)[1] == '\\') {
 		/* convert UNC paths (eg. \\server\sharepath) */
-		/* See http://blogs.msdn.com/ie/archive/2006/12/06/file-uris-in-windows.aspx */
+		/* See https://docs.microsoft.com/en-us/archive/blogs/ie/file-uris-in-windows */
 		tmp = xdebug_sprintf("file:%s", encoded_fileurl);
-	} else if (fileurl[0] == '/' || fileurl[0] == '\\') {
+	} else if (ZSTR_VAL(fileurl)[0] == '/' || ZSTR_VAL(fileurl)[0] == '\\') {
 		/* convert *nix paths (eg. /path) */
 		tmp = xdebug_sprintf("file://%s", encoded_fileurl);
-	} else if (fileurl[1] == ':') {
+	} else if (ZSTR_VAL(fileurl)[1] == ':') {
 		/* convert windows drive paths (eg. c:\path) */
 		tmp = xdebug_sprintf("file:///%s", encoded_fileurl);
 	} else {
@@ -448,7 +471,7 @@ lock: /* Yes yes, an evil goto label here!!! */
 	return fh;
 }
 #else
-FILE *xdebug_fopen(char *fname, char *mode, char *extension, char **new_fname)
+FILE *xdebug_fopen(char *fname, const char *mode, const char *extension, char **new_fname)
 {
 	char *tmp_fname;
 	FILE *ret;
@@ -468,6 +491,7 @@ FILE *xdebug_fopen(char *fname, char *mode, char *extension, char **new_fname)
 }
 #endif
 
+
 int xdebug_format_output_filename(char **filename, char *format, char *script_name)
 {
 	xdebug_str fname = XDEBUG_STR_INITIALIZER;
@@ -476,23 +500,23 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 	while (*format)
 	{
 		if (*format != '%') {
-			xdebug_str_addl(&fname, (char *) format, 1, 0);
+			xdebug_str_addc(&fname, *format);
 		} else {
 			format++;
 			switch (*format)
 			{
 				case 'c': /* crc32 of the current working directory */
 					if (VCWD_GETCWD(cwd, 127)) {
-						xdebug_str_add(&fname, xdebug_sprintf("%lu", xdebug_crc32(cwd, strlen(cwd))), 1);
+						xdebug_str_add_fmt(&fname, "%lu", xdebug_crc32(cwd, strlen(cwd)));
 					}
 					break;
 
 				case 'p': /* pid */
-					xdebug_str_add(&fname, xdebug_sprintf(ZEND_ULONG_FMT, xdebug_get_pid()), 1);
+					xdebug_str_add_fmt(&fname, ZEND_ULONG_FMT, xdebug_get_pid());
 					break;
 
 				case 'r': /* random number */
-					xdebug_str_add(&fname, xdebug_sprintf("%06x", (long) (1000000 * php_combined_lcg())), 1);
+					xdebug_str_add_fmt(&fname, "%06x", (long) (1000000 * php_combined_lcg()));
 					break;
 
 				case 's': { /* script fname */
@@ -522,19 +546,16 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 				}	break;
 
 				case 't': { /* timestamp (in seconds) */
-					time_t the_time = time(NULL);
-					xdebug_str_add(&fname, xdebug_sprintf("%ld", the_time), 1);
+					xdebug_str_add_fmt(&fname, "%lu", xdebug_get_nanotime() / NANOS_IN_SEC);
 				}	break;
 
 				case 'u': { /* timestamp (in microseconds) */
-					char *char_ptr, *utime_str = xdebug_sprintf("%F", xdebug_get_utime());
+					uint64_t nanotime = xdebug_get_nanotime();
 
-					/* Replace . with _ (or should it be nuked?) */
-					char_ptr = strrchr(utime_str, '.');
-					if (char_ptr) {
-						char_ptr[0] = '_';
-					}
-					xdebug_str_add(&fname, utime_str, 1);
+					xdebug_str_add_fmt(&fname, "%lu.%06d",
+						nanotime / NANOS_IN_SEC,
+						(nanotime % NANOS_IN_SEC) / NANOS_IN_MICROSEC
+					);
 				}	break;
 
 				case 'H':   /* $_SERVER['HTTP_HOST'] */
@@ -593,7 +614,7 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 				}	break;
 
 				case '%': /* literal % */
-					xdebug_str_addl(&fname, "%", 1, 0);
+					xdebug_str_addc(&fname, '%');
 					break;
 			}
 		}
@@ -608,26 +629,26 @@ int xdebug_format_output_filename(char **filename, char *format, char *script_na
 int xdebug_format_file_link(char **filename, const char *error_filename, int error_lineno)
 {
 	xdebug_str fname = XDEBUG_STR_INITIALIZER;
-	char      *format = XINI_BASE(file_link_format);
+	char      *format = XINI_LIB(file_link_format);
 
 	while (*format)
 	{
 		if (*format != '%') {
-			xdebug_str_addl(&fname, (char *) format, 1, 0);
+			xdebug_str_addc(&fname, *format);
 		} else {
 			format++;
 			switch (*format)
 			{
 				case 'f': /* filename */
-					xdebug_str_add(&fname, xdebug_sprintf("%s", error_filename), 1);
+					xdebug_str_add(&fname, error_filename, 0);
 					break;
 
 				case 'l': /* line number */
-					xdebug_str_add(&fname, xdebug_sprintf("%d", error_lineno), 1);
+					xdebug_str_add_fmt(&fname, "%d", error_lineno);
 					break;
 
 				case '%': /* literal % */
-					xdebug_str_addl(&fname, "%", 1, 0);
+					xdebug_str_addc(&fname, '%');
 					break;
 			}
 		}
@@ -639,19 +660,20 @@ int xdebug_format_file_link(char **filename, const char *error_filename, int err
 	return fname.l;
 }
 
-int xdebug_format_filename(char **formatted_name, const char *fmt, const char *default_fmt, const char *filename)
+int xdebug_format_filename(char **formatted_name, const char *default_fmt, zend_string *filename)
 {
 	xdebug_str fname = XDEBUG_STR_INITIALIZER;
 	char *name;
 	xdebug_str *parent, *ancester;
-	const char *full = filename;
-	xdebug_arg *parts = (xdebug_arg*) xdmalloc(sizeof(xdebug_arg));
+	const char *full_filename = ZSTR_VAL(filename);
+	xdebug_arg *parts;
 	char *slash = xdebug_sprintf("%c", DEFAULT_SLASH);
+	char *fmt = XINI_LIB(filename_format);
 	const char *format = fmt && fmt[0] ? fmt : default_fmt; /* If the format is empty, we use the default */
 
 	/* Create pointers for the format chars */
-	xdebug_arg_init(parts);
-	xdebug_explode(slash, (char*) filename, parts, -1);
+	parts = xdebug_arg_ctor();
+	xdebug_explode(slash, full_filename, parts, -1);
 	name = parts->args[parts->c - 1];
 	parent = parts->c > 1 ?
 		xdebug_join(slash, parts, parts->c - 2, parts->c - 1) :
@@ -663,28 +685,28 @@ int xdebug_format_filename(char **formatted_name, const char *fmt, const char *d
 	while (*format)
 	{
 		if (*format != '%') {
-			xdebug_str_addl(&fname, (char *) format, 1, 0);
+			xdebug_str_addc(&fname, *format);
 		} else {
 			format++;
 			switch (*format)
 			{
 				case 'n': /* filename */
-					xdebug_str_add(&fname, xdebug_sprintf("%s", name), 1);
+					xdebug_str_add(&fname, name, 0);
 					break;
 				case 'p': /* parent */
-					xdebug_str_add(&fname, xdebug_sprintf("%s", parent->d), 1);
+					xdebug_str_add_str(&fname, parent);
 					break;
 				case 'a': /* ancester */
-					xdebug_str_add(&fname, xdebug_sprintf("%s", ancester->d), 1);
+					xdebug_str_add_str(&fname, ancester);
 					break;
 				case 'f': /* full path */
-					xdebug_str_add(&fname, xdebug_sprintf("%s", full), 1);
+					xdebug_str_add(&fname, full_filename, 0);
 					break;
 				case 's': /* slash */
-					xdebug_str_add(&fname, xdebug_sprintf("%c", DEFAULT_SLASH), 1);
+					xdebug_str_addc(&fname, DEFAULT_SLASH);
 					break;
 				case '%': /* literal % */
-					xdebug_str_addl(&fname, "%", 1, 0);
+					xdebug_str_addc(&fname, '%');
 					break;
 			}
 		}
