@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include "zend.h"
@@ -59,7 +57,6 @@ typedef struct xdebug_var_export_options {
 #define XDEBUG_VAR_TYPE_STATIC   0x01
 #define XDEBUG_VAR_TYPE_CONSTANT 0x02
 
-void xdebug_dump_used_var_with_contents(void *htmlq, xdebug_hash_element* he, void *argument);
 void xdebug_get_php_symbol(zval *retval, xdebug_str* name);
 
 xdebug_var_export_options* xdebug_var_export_options_from_ini(void);
@@ -69,10 +66,13 @@ xdebug_var_export_options* xdebug_var_get_nolimit_options(void);
 xdebug_str* xdebug_get_property_type(zval* object, zval *val);
 #endif
 xdebug_str* xdebug_get_property_info(char *mangled_property, int mangled_len, const char **modifier, char **class_name);
+
+#define XDEBUG_VAR_OBJDEBUG_DEFAULT        0x00
+#define XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO  0x01
 #if PHP_VERSION_ID >= 70400
-HashTable *xdebug_objdebug_pp(zval **zval_pp);
+HashTable *xdebug_objdebug_pp(zval **zval_pp, int flags);
 #else
-HashTable *xdebug_objdebug_pp(zval **zval_pp, int *is_tmp);
+HashTable *xdebug_objdebug_pp(zval **zval_pp, int *is_tmp, int flags);
 void xdebug_var_maybe_destroy_ht(HashTable *ht, int is_temp);
 #endif
 
@@ -82,12 +82,15 @@ void xdebug_var_maybe_destroy_ht(HashTable *ht, int is_temp);
 void xdebug_add_variable_attributes(xdebug_str *str, zval *struc, zend_bool fancy);
 
 
-char* xdebug_xmlize(char *string, size_t len, size_t *newlen);
+char* xdebug_xmlize(char *s_string, size_t len, size_t *newlen);
 char* xdebug_error_type_simple(int type);
 char* xdebug_error_type(int type);
 zval *xdebug_get_zval(zend_execute_data *zdata, int node_type, const znode_op *node, int *is_var);
 zval *xdebug_get_zval_with_opline(zend_execute_data *zdata, const zend_op *opline, int node_type, const znode_op *node, int *is_var);
 
-char* xdebug_show_fname(xdebug_func t, int html, int flags);
+#define XDEBUG_SHOW_FNAME_DEFAULT          0
+#define XDEBUG_SHOW_FNAME_ALLOW_HTML    1<<1
+#define XDEBUG_SHOW_FNAME_IGNORE_SCOPE  1<<2
+char* xdebug_show_fname(xdebug_func t, int flags);
 
 #endif
