@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,12 +12,14 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   +----------------------------------------------------------------------+
  */
 
 #include "var_export_html.h"
 #include "lib_private.h"
+#include "Zend/zend_closures.h"
+#if PHP_VERSION_ID >= 80100
+# include "zend_enum.h"
+#endif
 
 ZEND_EXTERN_MODULE_GLOBALS(xdebug)
 
@@ -116,6 +118,36 @@ static int xdebug_object_element_export_html(zval *object, zval *zv_nptr, zend_u
 	return 0;
 }
 
+static void handle_closure(xdebug_str *str, zval *obj, int level)
+{
+	const zend_function *closure_function;
+
+	if (Z_TYPE_P(obj) != IS_OBJECT || !instanceof_function(Z_OBJCE_P(obj), zend_ce_closure)) {
+		return;
+	}
+
+#if PHP_VERSION_ID >= 80000
+	closure_function = zend_get_closure_method_def(Z_OBJ_P(obj));
+#else
+	closure_function = zend_get_closure_method_def(obj);
+#endif
+
+	xdebug_str_add_fmt(str, "%*s<i>virtual</i> 'closure' <font color='%s'>'", (level * 4) - 2, "", COLOR_STRING);
+
+	if (closure_function->common.scope) {
+		if (closure_function->common.fn_flags & ZEND_ACC_STATIC) {
+			xdebug_str_add_zstr(str, closure_function->common.scope->name);
+			xdebug_str_add_literal(str, "::");
+		} else {
+			xdebug_str_add_literal(str, "$this->");
+		}
+	}
+	xdebug_str_add_zstr(str, closure_function->common.function_name);
+
+	xdebug_str_add_literal(str, "'</font>\n");
+}
+
+
 void xdebug_var_export_html(zval **struc, xdebug_str *str, int level, int debug_zval, xdebug_var_export_options *options)
 {
 	HashTable *myht;
@@ -214,11 +246,42 @@ void xdebug_var_export_html(zval **struc, xdebug_str *str, int level, int debug_
 			}
 			break;
 
-		case IS_OBJECT:
+		case IS_OBJECT: {
+#if PHP_VERSION_ID >= 80100
+			zend_class_entry *ce = Z_OBJCE_P(*struc);
+			if (ce->ce_flags & ZEND_ACC_ENUM) {
+				zval *case_name_zval = zend_enum_fetch_case_name(Z_OBJ_P(*struc));
+				xdebug_str_add_fmt(
+					str, "<b>enum</b>(<i>%s::%s</i>)",
+					ZSTR_VAL(ce->name),
+					Z_STRVAL_P(case_name_zval)
+				);
+				if (ce->enum_backing_type != IS_UNDEF) {
+					zval *value = zend_enum_fetch_case_value(Z_OBJ_P(*struc));
+
+					if (ce->enum_backing_type == IS_LONG) {
+						xdebug_str_add_fmt(
+							str, " : <small>int</small> <font color='%s'>" XDEBUG_INT_FMT "</font>",
+							COLOR_LONG, Z_LVAL_P(value)
+						);
+					}
+
+					if (ce->enum_backing_type == IS_STRING) {
+						xdebug_str_add_fmt(
+							str, " : <small>string</small> <font color='%s'>'%s'</font> <i>(length=%d)</i>",
+							COLOR_STRING, Z_STRVAL_P(value), Z_STRLEN_P(value)
+						);
+					}
+				}
+				xdebug_str_addc(str, '\n');
+				break;
+			}
+#endif
+
 #if PHP_VERSION_ID >= 70400
-			myht = xdebug_objdebug_pp(struc);
+			myht = xdebug_objdebug_pp(struc, XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO);
 #else
-			myht = xdebug_objdebug_pp(struc, &is_temp);
+			myht = xdebug_objdebug_pp(struc, &is_temp, XDEBUG_VAR_OBJDEBUG_USE_DEBUGINFO);
 #endif
 			xdebug_str_add_fmt(str, "\n%*s", (level - 1) * 4, "");
 
@@ -228,6 +291,8 @@ void xdebug_var_export_html(zval **struc, xdebug_str *str, int level, int debug_
 				xdebug_str_add_literal(str, "</i>)");
 				xdebug_str_add_fmt(str, "[<i>%d</i>]\n", Z_OBJ_HANDLE_P(*struc));
 
+				handle_closure(str, *struc, level);
+
 				if (myht && (level <= options->max_depth)) {
 					options->runtime[level].current_element_nr = 0;
 					options->runtime[level].start_element_nr = 0;
@@ -255,6 +320,7 @@ void xdebug_var_export_html(zval **struc, xdebug_str *str, int level, int debug_
 			xdebug_var_maybe_destroy_ht(myht, is_temp);
 #endif
 			break;
+		}
 
 		case IS_RESOURCE: {
 			char *type_name;
@@ -361,9 +427,18 @@ static void xdebug_var_synopsis_html(zval **struc, xdebug_str *str, int level, i
 			xdebug_str_add_fmt(str, "<font color='%s'>array(%d)</font>", COLOR_ARRAY, myht->nNumOfElements);
 			break;
 
-		case IS_OBJECT:
+		case IS_OBJECT: {
+#if PHP_VERSION_ID >= 80100
+			zend_class_entry *ce = Z_OBJCE_P(*struc);
+			if (ce->ce_flags & ZEND_ACC_ENUM) {
+				zval *case_name_zval = zend_enum_fetch_case_name(Z_OBJ_P(*struc));
+				xdebug_str_add_fmt( str, "<font color='%s'>enum(%s::%s)</font>", COLOR_OBJECT, ZSTR_VAL(ce->name), Z_STRVAL_P(case_name_zval));
+				break;
+			}
+#endif
+
 			xdebug_str_add_fmt(str, "<font color='%s'>object(%s)[%d]</font>", COLOR_OBJECT, ZSTR_VAL(Z_OBJCE_P(*struc)->name), Z_OBJ_HANDLE_P(*struc));
-			break;
+		} break;
 
 		case IS_RESOURCE: {
 			char *type_name;
