@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2020 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,9 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors: Derick Rethans <derick@xdebug.org>                          |
-   |          Michael Voříšek <mvorisek@mvorisek.cz>                      |
-   +----------------------------------------------------------------------+
  */
 
 #ifndef __XDEBUG_PROFILER_H__
@@ -22,18 +19,20 @@
 
 #include "php.h"
 #include "TSRM.h"
+#include "lib/file.h"
 #include "lib/lib.h"
 
+#include "php_xdebug.h"
+
 typedef struct _xdebug_profiler_globals_t {
-	zend_bool     active;
-	uint64_t      profiler_start_nanotime;
-	FILE         *profile_file;
-	char         *profile_filename;
-	xdebug_hash  *profile_filename_refs;
-	int           profile_last_filename_ref;
-	int           php_internal_seen_before;
-	xdebug_hash  *profile_functionname_refs;
-	int           profile_last_functionname_ref;
+	zend_bool       active;
+	uint64_t        profiler_start_nanotime;
+	xdebug_file     profile_file;
+	xdebug_hash    *profile_filename_refs;
+	int             profile_last_filename_ref;
+	int             php_internal_seen_before;
+	xdebug_hash    *profile_functionname_refs;
+	int             profile_last_functionname_ref;
 } xdebug_profiler_globals_t;
 
 typedef struct _xdebug_profiler_settings_t {
