@@ -2,7 +2,7 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2018 Derick Rethans                               |
+   | Copyright (c) 2002-2021 Derick Rethans                               |
    +----------------------------------------------------------------------+
    | This source file is subject to version 1.01 of the Xdebug license,   |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,8 +12,6 @@
    | to obtain it through the world-wide-web, please send a note to       |
    | derick@xdebug.org so we can mail you a copy immediately.             |
    +----------------------------------------------------------------------+
-   | Authors:  Derick Rethans <derick@xdebug.org>                         |
-   +----------------------------------------------------------------------+
  */
 #ifndef XDEBUG_TRACE_TEXTUAL_H
 #define XDEBUG_TRACE_TEXTUAL_H
@@ -22,8 +20,7 @@
 
 typedef struct _xdebug_trace_textual_context
 {
-	FILE *trace_file;
-	char *trace_filename;
+	xdebug_file *trace_file;
 } xdebug_trace_textual_context;
 
 extern xdebug_trace_handler_t xdebug_trace_handler_textual;
