@@ -2,15 +2,15 @@
    +----------------------------------------------------------------------+
    | Xdebug                                                               |
    +----------------------------------------------------------------------+
-   | Copyright (c) 2002-2018 Derick Rethans                               |
+   | Copyright (c) 2002-2016 Derick Rethans                               |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 1.01 of the Xdebug license,   |
+   | This source file is subject to version 1.0 of the Xdebug license,    |
    | that is bundled with this package in the file LICENSE, and is        |
    | available at through the world-wide-web at                           |
-   | https://xdebug.org/license.php                                       |
+   | http://xdebug.derickrethans.nl/license.php                           |
    | If you did not receive a copy of the Xdebug license and are unable   |
    | to obtain it through the world-wide-web, please send a note to       |
-   | derick@xdebug.org so we can mail you a copy immediately.             |
+   | xdebug@derickrethans.nl so we can mail you a copy immediately.       |
    +----------------------------------------------------------------------+
    | Authors: Derick Rethans <derick@xdebug.org>                          |
    +----------------------------------------------------------------------+
@@ -39,7 +39,7 @@ xdebug_trace_handler_t *xdebug_select_trace_handler(int options TSRMLS_DC)
 		case 1: tmp = &xdebug_trace_handler_computerized; break;
 		case 2: tmp = &xdebug_trace_handler_html; break;
 		default:
-			php_error(E_NOTICE, "A wrong value for xdebug.trace_format was selected (%d), defaulting to the textual format", (int) XG(trace_format));
+			php_error(E_NOTICE, "A wrong value for xdebug.trace_format was selected (%d), defaulting to the textual format.", (int) XG(trace_format));
 			tmp = &xdebug_trace_handler_textual; break;
 	}
 
@@ -53,7 +53,7 @@ xdebug_trace_handler_t *xdebug_select_trace_handler(int options TSRMLS_DC)
 	return tmp;
 }
 
-FILE *xdebug_trace_open_file(char *fname, char *script_filename, long options, char **used_fname TSRMLS_DC)
+FILE *xdebug_trace_open_file(char *fname, long options, char **used_fname TSRMLS_DC)
 {
 	FILE *file;
 	char *filename;
@@ -62,7 +62,7 @@ FILE *xdebug_trace_open_file(char *fname, char *script_filename, long options, c
 		filename = xdstrdup(fname);
 	} else {
 		if (!strlen(XG(trace_output_name)) ||
-			xdebug_format_output_filename(&fname, XG(trace_output_name), script_filename) <= 0
+			xdebug_format_output_filename(&fname, XG(trace_output_name), NULL) <= 0
 		) {
 			/* Invalid or empty xdebug.trace_output_name */
 			return NULL;
@@ -84,16 +84,13 @@ FILE *xdebug_trace_open_file(char *fname, char *script_filename, long options, c
 	return file;
 }
 
-char* xdebug_start_trace(char* fname, char *script_filename, long options TSRMLS_DC)
+char* xdebug_start_trace(char* fname, long options TSRMLS_DC)
 {
-	if (XG(trace_context)) {
-		return NULL;
-	}
-
 	XG(trace_handler) = xdebug_select_trace_handler(options TSRMLS_CC);
-	XG(trace_context) = (void*) XG(trace_handler)->init(fname, script_filename, options TSRMLS_CC);
+	XG(trace_context) = (void*) XG(trace_handler)->init(fname, options TSRMLS_CC);
 
 	if (XG(trace_context)) {
+		XG(do_trace) = 1;
 		XG(trace_handler)->write_header(XG(trace_context) TSRMLS_CC);
 		return xdstrdup(XG(trace_handler)->get_filename(XG(trace_context) TSRMLS_CC));
 	}
@@ -103,6 +100,7 @@ char* xdebug_start_trace(char* fname, char *script_filename, long options TSRMLS
 
 void xdebug_stop_trace(TSRMLS_D)
 {
+	XG(do_trace) = 0;
 	if (XG(trace_context)) {
 		XG(trace_handler)->write_footer(XG(trace_context) TSRMLS_CC);
 		XG(trace_handler)->deinit(XG(trace_context) TSRMLS_CC);
@@ -113,27 +111,33 @@ void xdebug_stop_trace(TSRMLS_D)
 PHP_FUNCTION(xdebug_start_trace)
 {
 	char *fname = NULL;
+#if PHP_VERSION_ID >= 70000
 	size_t fname_len = 0;
+#else
+	int   fname_len = 0;
+#endif
 	char *trace_fname;
-	zend_long options = XG(trace_options);
-
-	if (!XG(trace_context)) {
-		function_stack_entry *fse;
+	zppLONG options = XG(trace_options);
 
+	if (XG(do_trace) == 0) {
 		if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "|sl", &fname, &fname_len, &options) == FAILURE) {
 			return;
 		}
 
-		fse = xdebug_get_stack_frame(0 TSRMLS_CC);
-
-		if ((trace_fname = xdebug_start_trace(fname, fse->filename, options TSRMLS_CC)) != NULL) {
+		if ((trace_fname = xdebug_start_trace(fname, options TSRMLS_CC)) != NULL) {
+			XG(do_trace) = 1;
+#if PHP_VERSION_ID >= 70000
 			RETVAL_STRING(trace_fname);
+#else
+			RETVAL_STRING(trace_fname, 1);
+#endif
 			xdfree(trace_fname);
 			return;
 		} else {
 			php_error(E_NOTICE, "Trace could not be started");
 		}
 
+		XG(do_trace) = 0;
 		RETURN_FALSE;
 	} else {
 		php_error(E_NOTICE, "Function trace already started");
@@ -143,8 +147,12 @@ PHP_FUNCTION(xdebug_start_trace)
 
 PHP_FUNCTION(xdebug_stop_trace)
 {
-	if (XG(trace_context)) {
+	if (XG(do_trace) == 1) {
+#if PHP_VERSION_ID >= 70000
 		RETVAL_STRING(XG(trace_handler)->get_filename(XG(trace_context) TSRMLS_CC));
+#else
+		RETVAL_STRING(XG(trace_handler)->get_filename(XG(trace_context) TSRMLS_CC), 1);
+#endif
 		xdebug_stop_trace(TSRMLS_C);
 	} else {
 		RETVAL_FALSE;
@@ -154,8 +162,12 @@ PHP_FUNCTION(xdebug_stop_trace)
 
 PHP_FUNCTION(xdebug_get_tracefile_name)
 {
-	if (XG(trace_context) && XG(trace_handler) && XG(trace_handler)->get_filename) {
+	if (XG(do_trace) == 1 && XG(trace_handler) && XG(trace_handler)->get_filename) {
+#if PHP_VERSION_ID >= 70000
 		RETVAL_STRING(XG(trace_handler)->get_filename(XG(trace_context) TSRMLS_CC));
+#else
+		RETVAL_STRING(XG(trace_handler)->get_filename(XG(trace_context) TSRMLS_CC), 1);
+#endif
 	} else {
 		RETURN_FALSE;
 	}
